/* -*- C++ -*- vim:set syntax=cpp:
 * (C) Frankt-Rene Schaefer
 * ABSOLUTELY NO WARRANTY               */
#ifndef QUEX_INCLUDE_GUARD_TestAnalyzer__ANALYZER__MEMBER__MODE_HANDLING_I
#define QUEX_INCLUDE_GUARD_TestAnalyzer__ANALYZER__MEMBER__MODE_HANDLING_I

#include "test_c/lib/definitions"
#include "test_c/lib/analyzer/Mode"



 const TestAnalyzer_Mode*
TestAnalyzer_MF_mode(struct TestAnalyzer_tag* me) 
{ return me->__current_mode_p; }

 void 
TestAnalyzer_MF_set_mode_brutally(struct TestAnalyzer_tag* me, const TestAnalyzer_Mode* ModeP) 
/* Core of all mode setting functions. 
 *
 * ADAPTS: -- current mode pointer.
 *         -- current analyzer function pointer
 *         -- setting the buffer's handlers for 'on_buffer_overflow' and 
 *            'on_buffer_before_change'                                   */
{ 
    __quex_debug_show_mode_transition(me, ModeP);

    me->__current_mode_p          = ModeP;
    me->current_analyzer_function = ModeP->analyzer_function; 

    TestAnalyzer_Buffer_callbacks_set(&me->buffer, 
                                    me->__current_mode_p->buffer_callbacks.on_buffer_before_change,
                                    me->__current_mode_p->buffer_callbacks.on_buffer_overflow,
                                    (void*)me);
}

 void    
TestAnalyzer_MF_enter_mode(struct TestAnalyzer_tag* me, const TestAnalyzer_Mode* TargetMode) 
{

    const TestAnalyzer_Mode* SourceMode = me->__current_mode_p;



    me->__current_mode_p->on_exit(me, TargetMode);


#   ifdef QUEX_OPTION_ASSERTS
    if( ! me->__current_mode_p->has_exit_to(TargetMode) ) {
        QUEX_DEBUG_PRINT2("Mode '%s' has no exit to '%s'.\n", 
                          me->__current_mode_p->name,
                          TargetMode->name);
        TestAnalyzer_MF_error_code_set_if_first(me, E_Error_ModeInadmissibleTransition);
    }
    if( ! TargetMode->has_entry_from(me->__current_mode_p) ) {
        QUEX_DEBUG_PRINT2("Mode '%s' has no entry from '%s'.\n", 
                          TargetMode->name,
                          me->__current_mode_p->name);
        TestAnalyzer_MF_error_code_set_if_first(me, E_Error_ModeInadmissibleTransition);
    }
#   endif

    TestAnalyzer_MF_set_mode_brutally(me, TargetMode);


    TargetMode->on_entry(me, SourceMode);         

}

 void 
TestAnalyzer_MF_pop_mode(struct TestAnalyzer_tag* me) 
{ 
    if( me->_mode_stack.end == me->_mode_stack.begin ) {
        TestAnalyzer_MF_error_code_set_if_first(me, E_Error_ModeStack_PopOnTopLevel);
    }
    else {
        --(me->_mode_stack.end);
        TestAnalyzer_MF_enter_mode(me, *me->_mode_stack.end); 
    }
}

 void
TestAnalyzer_MF_pop_drop_mode(struct TestAnalyzer_tag* me) 
{ 
    if( me->_mode_stack.end == me->_mode_stack.begin ) {
        TestAnalyzer_MF_error_code_set_if_first(me, E_Error_ModeStack_PopOnTopLevel);
    }
    else {
        --(me->_mode_stack.end);
    }
}
    
 void       
TestAnalyzer_MF_push_mode(struct TestAnalyzer_tag* me, TestAnalyzer_Mode* new_mode) 
{ 
    if( me->_mode_stack.end == me->_mode_stack.memory_end ) {
        TestAnalyzer_MF_error_code_set_if_first(me, E_Error_ModeStack_Overflow);
    }
    else {
        *me->_mode_stack.end = me->__current_mode_p;
        ++(me->_mode_stack.end);
        TestAnalyzer_MF_enter_mode(me, new_mode); 
    }
}



#endif /* QUEX_INCLUDE_GUARD_TestAnalyzer__ANALYZER__MEMBER__MODE_HANDLING_I */
