/* vim: ft=c:
 * (C) Frank-Rene Schaefer */
#ifndef  QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_OSAL_I
#define  QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_OSAL_I

#include "test_c/lib/quex/MemoryManager"
#include "test_c/lib/quex/byte_loader/ByteLoader_OSAL"

#include <osapi.h>     /* Usually located in directory '$OSAL_SRC/src/os/inc' */



extern void    quex_ByteLoader_OSAL_chunk_seek(quex_ByteLoader* me, 
                                                   QUEX_TYPE_STREAM_POSITION Pos);
extern size_t  quex_ByteLoader_OSAL_chunk_load(quex_ByteLoader* me, 
                                                   void* buffer, const size_t ByteN, 
                                                   bool*);
extern void    quex_ByteLoader_OSAL_destruct(quex_ByteLoader* me);
extern void    quex_ByteLoader_OSAL_print_this(quex_ByteLoader* me);
extern bool    quex_ByteLoader_OSAL_compare_handle(const quex_ByteLoader* alter_ego_A, 
                                                             const quex_ByteLoader* alter_ego_B);
extern bool    quex_ByteLoader_OSAL_descriptor_good(int32 fd);



quex_ByteLoader*    
quex_ByteLoader_OSAL_new(int32 fd)
{
    quex_ByteLoader_OSAL* me;

    if( ! quex_ByteLoader_OSAL_descriptor_good(fd) ) return (quex_ByteLoader*)0;
    me = (quex_ByteLoader_OSAL*)quex_MemoryManager_allocate(sizeof(quex_ByteLoader_OSAL),
                                                           E_MemoryObjectType_BYTE_LOADER);
    if( ! me ) {
        return (quex_ByteLoader*)0;
    }
    else if( ! quex_ByteLoader_OSAL_construct(me, fd) ) {
        quex_MemoryManager_free(me, E_MemoryObjectType_BYTE_LOADER);
        return (quex_ByteLoader*)0;
    }
    else {
        (me->base).ownership = E_Ownership_LEXER;
        return &(me->base);
    }
}

quex_ByteLoader*    
quex_ByteLoader_OSAL_new_from_file_name(const char* FileName)
{
    quex_ByteLoader_OSAL* me;

    me = (quex_ByteLoader_OSAL*)quex_MemoryManager_allocate(sizeof(quex_ByteLoader_OSAL),
                                                           E_MemoryObjectType_BYTE_LOADER);
    if( ! me ) {
        return (quex_ByteLoader*)0;
    }
    else if( ! quex_ByteLoader_OSAL_construct_from_file_name(me, FileName) ) {
        quex_MemoryManager_free(me, E_MemoryObjectType_BYTE_LOADER);
        return (quex_ByteLoader*)0;
    }
    else {
        (me->base).ownership = E_Ownership_LEXER;
        return &(me->base);
    }
}

bool
quex_ByteLoader_OSAL_construct_from_file_name(quex_ByteLoader_OSAL* me, 
                                                        const char*                       FileName)
{
    int32 fd = OS_open(FileName, OS_READ_ONLY, 0x644);
    return quex_ByteLoader_OSAL_construct(me, fd);
}

bool
quex_ByteLoader_OSAL_construct(quex_ByteLoader_OSAL* me, int32 fd)
{
    QUEX_TYPE_STREAM_POSITION chunk_position_zero = quex_ByteLoader_OSAL_descriptor_good(fd) ? 
                                                      (QUEX_TYPE_STREAM_POSITION)OS_lseek(fd, 0, SEEK_CUR) 
                                                    : (QUEX_TYPE_STREAM_POSITION)0;
    me->fd = fd;

    /* A OSAL file handle is always in binary mode.                         */
    quex_ByteLoader_construct(&(me->base), true,
                                    /* ChunkSizeInBytes */ 1,
                                    /* ChunkPositionOfReadByteIdxZero */ chunk_position_zero,
                                    quex_ByteLoader_OSAL_chunk_seek,
                                    quex_ByteLoader_OSAL_chunk_load,
                                    quex_ByteLoader_OSAL_destruct,
                                    quex_ByteLoader_OSAL_print_this,
                                    quex_ByteLoader_OSAL_compare_handle);

    return quex_ByteLoader_OSAL_descriptor_good(fd);
}

void    
quex_ByteLoader_OSAL_destruct(quex_ByteLoader* alter_ego)
{
    quex_ByteLoader_OSAL* me = (quex_ByteLoader_OSAL*)(alter_ego);

    if( quex_ByteLoader_OSAL_descriptor_good(me->fd) ) {
        OS_close(me->fd);
        me->fd = OS_FS_ERROR;
    }
}

void                      
quex_ByteLoader_OSAL_chunk_seek(quex_ByteLoader*  alter_ego, 
                                    QUEX_TYPE_STREAM_POSITION    Pos) 
{ 
    quex_ByteLoader_OSAL* me = (quex_ByteLoader_OSAL*)(alter_ego);
    if( ! quex_ByteLoader_OSAL_descriptor_good(me->fd) ) { return; }
    (void)OS_lseek(me->fd, (long)Pos, SEEK_SET); 
}

size_t  
quex_ByteLoader_OSAL_chunk_load(quex_ByteLoader* alter_ego, 
                                    void*                       buffer, 
                                    const size_t                ByteN, 
                                    bool*                       end_of_stream_f) 
/* The OSAL interface does not allow to detect end of file upon reading.
 * The caller will realize end of stream by a return of zero bytes.          */
{ 
    quex_ByteLoader_OSAL* me = (quex_ByteLoader_OSAL*)(alter_ego);
    int32                            n;

    if( ! quex_ByteLoader_OSAL_descriptor_good(me->fd) ) { *end_of_stream_f = true; return 0; }

    n = OS_read(me->fd, buffer, ByteN); 
    switch( n ) {
    case OS_FS_ERR_INVALID_POINTER: n = 0; break;
    case OS_FS_ERROR:               n = 0; break;
    default:                        n = (n>=OS_SUCCESS) ? n : 0; break;
    }

    /* Theoretically, a last 'terminating zero' might be send over socket 
     * connections. Make sure, that this does not appear in the stream.      */
    if( n && ((uint8_t*)buffer)[n-1] == 0x0 ) {
        --n;
    }
    *end_of_stream_f = false;
    return (size_t)n;
}

bool  
quex_ByteLoader_OSAL_compare_handle(const quex_ByteLoader* alter_ego_A, 
                                              const quex_ByteLoader* alter_ego_B) 
/* RETURNS: true  -- if A and B point to the same OSAL object.
 *          false -- else.                                                   */
{ 
    const quex_ByteLoader_OSAL* A = (quex_ByteLoader_OSAL*)(alter_ego_A);
    const quex_ByteLoader_OSAL* B = (quex_ByteLoader_OSAL*)(alter_ego_B);

    return A->fd == B->fd;
}

bool    
quex_ByteLoader_OSAL_descriptor_good(int32 fd)
{
    switch( fd ) {
    case OS_FS_ERROR:                 return false;
    case OS_FS_ERR_DEVICE_NOT_FREE:   return false;
    case OS_FS_ERR_DRIVE_NOT_CREATED: return false;
    case OS_FS_ERR_INVALID_POINTER:   return false;
    case OS_FS_ERR_NAME_TOO_LONG:     return false;
    case OS_FS_ERR_NO_FREE_FDS:       return false;
    case OS_FS_ERR_PATH_INVALID:      return false;
    case OS_FS_ERR_PATH_TOO_LONG:     return false;
    default:                          return fd >= OS_SUCCESS ? true : false;
    }
}


void                       
quex_ByteLoader_OSAL_print_this(quex_ByteLoader* alter_ego)
{
    quex_ByteLoader_OSAL* me = (quex_ByteLoader_OSAL*)(alter_ego);
    (void)me; (void)alter_ego;

    QUEX_DEBUG_PRINT("        type:             OSAL;\n");
    QUEX_DEBUG_PRINT1("        file_descriptor:  ((%i));\n", (int32)me->fd);
    QUEX_DEBUG_PRINT("        end_of_stream_f:  <no means to detect>;\n");
}



#endif /*  QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_OSAL_I */

