/* vim: ft=c:
 * (C) Frank-Rene Schaefer */
#ifndef  QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_POSIX_I
#define  QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_POSIX_I

#include "test_c/lib/quex/MemoryManager"
#include "test_c/lib/quex/byte_loader/ByteLoader_POSIX"



extern void                       quex_ByteLoader_POSIX_chunk_seek(quex_ByteLoader* me, 
                                                                       QUEX_TYPE_STREAM_POSITION Pos);
extern size_t                     quex_ByteLoader_POSIX_chunk_load(quex_ByteLoader* me, 
                                                                       void* buffer, const size_t ByteN, 
                                                                       bool*);
extern void                       quex_ByteLoader_POSIX_destruct(quex_ByteLoader* me);
extern void                       quex_ByteLoader_POSIX_print_this(quex_ByteLoader* me);
extern bool                       quex_ByteLoader_POSIX_compare_handle(const quex_ByteLoader* alter_ego_A, 
                                                                       const quex_ByteLoader* alter_ego_B);



quex_ByteLoader*    
quex_ByteLoader_POSIX_new(int fd)
{
    quex_ByteLoader_POSIX* me;

    if( fd == -1 ) return (quex_ByteLoader*)0;
    me = (quex_ByteLoader_POSIX*)quex_MemoryManager_allocate(sizeof(quex_ByteLoader_POSIX),
                                                           E_MemoryObjectType_BYTE_LOADER);
    if( ! me ) {
        return (quex_ByteLoader*)0;
    }
    else if( ! quex_ByteLoader_POSIX_construct(me, fd) ) {
        quex_MemoryManager_free(me, E_MemoryObjectType_BYTE_LOADER);
        return (quex_ByteLoader*)0;
    }
    else {
        (me->base).ownership = E_Ownership_LEXER;
        return &(me->base);
    }
}

quex_ByteLoader*    
quex_ByteLoader_POSIX_new_from_file_name(const char* FileName)
{
    quex_ByteLoader_POSIX* me;

    me = (quex_ByteLoader_POSIX*)quex_MemoryManager_allocate(sizeof(quex_ByteLoader_POSIX),
                                                           E_MemoryObjectType_BYTE_LOADER);
    if( ! me ) {
        return (quex_ByteLoader*)0;
    }
    else if( ! quex_ByteLoader_POSIX_construct_from_file_name(me, FileName) ) {
        quex_MemoryManager_free(me, E_MemoryObjectType_BYTE_LOADER);
        return (quex_ByteLoader*)0;
    }
    else {
        (me->base).ownership = E_Ownership_LEXER;
        return &(me->base);
    }
}

bool
quex_ByteLoader_POSIX_construct_from_file_name(quex_ByteLoader_POSIX* me, 
                                                         const char*                       FileName)
{
    int fd = open(FileName, O_RDONLY);
    return quex_ByteLoader_POSIX_construct(me, fd);
}

bool
quex_ByteLoader_POSIX_construct(quex_ByteLoader_POSIX* me, int fd)
{
    QUEX_TYPE_STREAM_POSITION chunk_position_zero = (fd != -1) ? (QUEX_TYPE_STREAM_POSITION)lseek(fd, 0, SEEK_CUR) 
                                                               : (QUEX_TYPE_STREAM_POSITION)0;
    me->fd = fd;

    /* A POSIX file handle is always in binary mode.                         */
    quex_ByteLoader_construct(&(me->base), true,
                                    /* ChunkSizeInBytes */ 1,
                                    /* ChunkPositionOfReadByteIdxZero */ chunk_position_zero,
                                    quex_ByteLoader_POSIX_chunk_seek,
                                    quex_ByteLoader_POSIX_chunk_load,
                                    quex_ByteLoader_POSIX_destruct,
                                    quex_ByteLoader_POSIX_print_this,
                                    quex_ByteLoader_POSIX_compare_handle);

    return fd == -1 ?  false : true;
}

void    
quex_ByteLoader_POSIX_destruct(quex_ByteLoader* alter_ego)
{
    quex_ByteLoader_POSIX* me = (quex_ByteLoader_POSIX*)(alter_ego);

    if( me->fd != -1 ) {
        close(me->fd);
        me->fd = -1;
    }
}

void                      
quex_ByteLoader_POSIX_chunk_seek(quex_ByteLoader*    alter_ego, 
                                 QUEX_TYPE_STREAM_POSITION Pos) 
{ 
    quex_ByteLoader_POSIX* me = (quex_ByteLoader_POSIX*)(alter_ego);
    if( me->fd == -1 ) { return; }
    (void)lseek(me->fd, (long)Pos, SEEK_SET); 
}

size_t  
quex_ByteLoader_POSIX_chunk_load(quex_ByteLoader* alter_ego, 
                                     void*                       buffer, 
                                     const size_t                ByteN, 
                                     bool*                       end_of_stream_f) 
/* The POSIX interface does not allow to detect end of file upon reading.
 * The caller will realize end of stream by a return of zero bytes.          */
{ 
    quex_ByteLoader_POSIX* me = (quex_ByteLoader_POSIX*)(alter_ego);
    ssize_t                      n;

    if( me->fd == -1 ) { *end_of_stream_f = true; return 0; }

    n = read(me->fd, buffer, ByteN); 

    /* Theoretically, a last 'terminating zero' might be send over socket 
     * connections. Make sure, that this does not appear in the stream.      */
    if( n && ((uint8_t*)buffer)[n-1] == 0x0 ) {
        --n;
    }
    *end_of_stream_f = false;
    return (size_t)n;
}

bool  
quex_ByteLoader_POSIX_compare_handle(const quex_ByteLoader* alter_ego_A, 
                                           const quex_ByteLoader* alter_ego_B) 
/* RETURNS: true  -- if A and B point to the same POSIX object.
 *          false -- else.                                                   */
{ 
    const quex_ByteLoader_POSIX* A = (quex_ByteLoader_POSIX*)(alter_ego_A);
    const quex_ByteLoader_POSIX* B = (quex_ByteLoader_POSIX*)(alter_ego_B);

    return A->fd == B->fd;
}

void                       
quex_ByteLoader_POSIX_print_this(quex_ByteLoader* alter_ego)
{
    quex_ByteLoader_POSIX* me = (quex_ByteLoader_POSIX*)(alter_ego);

    QUEX_DEBUG_PRINT("        type:             POSIX;\n");
    QUEX_DEBUG_PRINT1("        file_descriptor:  ((%i));\n", (int)me->fd);
    QUEX_DEBUG_PRINT("        end_of_stream_f:  <no means to detect>;\n");
}



#endif /*  QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_POSIX_I */

