/* vim:set ft=c: -*- C++ -*- */
#ifndef QUEX_INCLUDE_GUARD_TestAnalyzer__BUFFER__BUFFER_I
#define QUEX_INCLUDE_GUARD_TestAnalyzer__BUFFER__BUFFER_I

#include "test_c_cg/lib/buffer/asserts"
#include "test_c_cg/lib/definitions"
#include "test_c_cg/lib/buffer/Buffer"
#include "test_c_cg/lib/buffer/Buffer_print.i"
#include "test_c_cg/lib/buffer/lexatoms/LexatomLoader"
#include "test_c_cg/lib/quex/asserts"
#include "test_c_cg/lib/quex/MemoryManager"



 void      TestAnalyzer_Buffer_init_content(TestAnalyzer_Buffer* me, 
                                                     TestAnalyzer_lexatom_t* EndOfFileP);
 void      TestAnalyzer_Buffer_init_content_core(TestAnalyzer_Buffer*        me, 
                                                          QUEX_TYPE_STREAM_POSITION LI_Begin,
                                                          QUEX_TYPE_STREAM_POSITION LI_EndOfStream,
                                                          TestAnalyzer_lexatom_t*        EndOfFileP);
 void      TestAnalyzer_Buffer_init_analyzis(TestAnalyzer_Buffer*   me);
 void      TestAnalyzer_Buffer_init_analyzis_core(TestAnalyzer_Buffer*        me,
                                                           TestAnalyzer_lexatom_t*        ReadP,
                                                           TestAnalyzer_lexatom_t*        LexatomStartP,
                                                           TestAnalyzer_lexatom_t         LexatomAtLexemeStart,
                                                           TestAnalyzer_lexatom_t         LexatomBeforeLexemeStart,
                                                           QUEX_TYPE_STREAM_POSITION BackupLexatomIndexOfReadP);

 void      TestAnalyzer_Buffer_on_overflow_DEFAULT(void*              aux, 
                                                            TestAnalyzer_Buffer* buffer, 
                                                            bool               ForwardF);
 void      TestAnalyzer_Buffer_on_before_buffer_change_DEFAULT(void*  aux,
                                                                        const  TestAnalyzer_lexatom_t*, 
                                                                        const  TestAnalyzer_lexatom_t*);
 void      TestAnalyzer_Buffer_member_functions_assign(TestAnalyzer_Buffer* me);
#ifdef QUEX_OPTION_DEBUG_SHOW_EXT
 void      TestAnalyzer_Buffer_brief_trace_output(const TestAnalyzer_Buffer*  me,
                                                           TestAnalyzer_lexatom_t**       position_register, 
                                                           const size_t              PositionRegisterN);
#endif

 void
TestAnalyzer_Buffer_construct(TestAnalyzer_Buffer*        me, 
                            TestAnalyzer_LexatomLoader* filler,
                            TestAnalyzer_lexatom_t*        memory,
                            const size_t              MemorySize,
                            TestAnalyzer_lexatom_t*        EndOfFileP,
                            const ptrdiff_t           FallbackN,
                            E_Ownership               Ownership,
                            TestAnalyzer_Buffer*        IncludingBuffer)
{
    __quex_assert(QUEX_TestAnalyzer_SETTING_BUFFER_SIZE_MIN <= QUEX_TestAnalyzer_SETTING_BUFFER_SIZE);

    /* Ownership of InputMemory is passed to 'me->_memory'.                  */
    TestAnalyzer_BufferMemory_construct(&me->_memory, memory, MemorySize, 
                                      Ownership, IncludingBuffer); 
    
    me->filler = filler;
    TestAnalyzer_Buffer_member_functions_assign(me);

    /* Event handlers.                                                       */
    TestAnalyzer_Buffer_callbacks_set(me, (void (*)(void*))0, (void (*)(void*))0, (void*)0);

    /* Initialize.                                                           */
    TestAnalyzer_Buffer_init(me, EndOfFileP);
    me->_fallback_n = FallbackN;

    TestAnalyzer_Buffer_assert_consistency(me);
}

 void
TestAnalyzer_Buffer_member_functions_assign(TestAnalyzer_Buffer* me)
{
    me->fill                = TestAnalyzer_Buffer_fill;
    me->fill_prepare        = TestAnalyzer_Buffer_fill_prepare;
    me->fill_finish         = TestAnalyzer_Buffer_fill_finish;

    me->begin               = TestAnalyzer_Buffer_memory_begin;
    me->end                 = TestAnalyzer_Buffer_memory_end;
    me->size                = TestAnalyzer_Buffer_memory_size;

    me->content_space_end   = TestAnalyzer_Buffer_memory_content_space_end;
    me->content_space_size  = TestAnalyzer_Buffer_memory_content_space_size;

    me->content_begin       = TestAnalyzer_Buffer_memory_content_begin;
    me->content_end         = TestAnalyzer_Buffer_memory_content_end;
    me->content_size        = TestAnalyzer_Buffer_memory_content_size;

#   if defined(QUEX_OPTION_DEBUG_SHOW_EXT)
    me->brief_trace_output   = TestAnalyzer_Buffer_brief_trace_output;
#   endif
}

 TestAnalyzer_lexatom_t*   TestAnalyzer_Buffer_memory_begin(const TestAnalyzer_Buffer* me) { return me->_memory._front; }
 TestAnalyzer_lexatom_t*   TestAnalyzer_Buffer_memory_end(const TestAnalyzer_Buffer* me)   { return &me->_memory._back[1]; }
 ptrdiff_t            TestAnalyzer_Buffer_memory_size(const TestAnalyzer_Buffer* me)  { return &me->_memory._back[1] - me->_memory._front; }

 TestAnalyzer_lexatom_t*   TestAnalyzer_Buffer_memory_content_space_end(const TestAnalyzer_Buffer* me)   { return me->_memory._back; }
 ptrdiff_t            TestAnalyzer_Buffer_memory_content_space_size(const TestAnalyzer_Buffer* me)  { return me->_memory._back - &me->_memory._front[1]; }

 TestAnalyzer_lexatom_t*   TestAnalyzer_Buffer_memory_content_begin(const TestAnalyzer_Buffer* me) { return &me->_memory._front[1]; }
 TestAnalyzer_lexatom_t*   TestAnalyzer_Buffer_memory_content_end(const TestAnalyzer_Buffer* me)   { return me->input.end_p; }
 ptrdiff_t            TestAnalyzer_Buffer_memory_content_size(const TestAnalyzer_Buffer* me)  { return me->input.end_p - &me->_memory._front[1]; }

 void
TestAnalyzer_Buffer_init(TestAnalyzer_Buffer* me, TestAnalyzer_lexatom_t* EndOfFileP)
{
    TestAnalyzer_Buffer_init_content(me, EndOfFileP);
    TestAnalyzer_Buffer_init_analyzis(me); 
}

 void
TestAnalyzer_Buffer_destruct(TestAnalyzer_Buffer* me)
/* Destruct 'me' and mark all resources as absent.                            */
{
    if( TestAnalyzer_Buffer_resources_absent(me) ) {
        return;
    }
    TestAnalyzer_Buffer_callbacks_on_buffer_before_change(me);

    if( me->filler ) {
        me->filler->destruct(me->filler); 
        quex_MemoryManager_free((void*)me->filler, E_MemoryObjectType_BUFFER_FILLER);
    }
    TestAnalyzer_BufferMemory_destruct(&me->_memory);
    TestAnalyzer_Buffer_resources_absent_mark(me);
}

 void
TestAnalyzer_Buffer_shallow_copy(TestAnalyzer_Buffer* drain, const TestAnalyzer_Buffer* source)
/*    ,.
 *   /  \   DANGER: Do not use this function, except that you totally 
 *  /    \                understand its implications!
 *  '----'
 * Copy indices and references *as is*. The purpose of this function is solely
 * to copy the setup of a buffer to a safe place and restore it. 
 *
 *           NOT TO BE USED AS A REPLACEMENT FOR COPYING/CLONING!
 *
 * At the time of this writing, the only propper application is when a backup 
 * is generated in a memento. When it is restored in the 'real' buffer object 
 * the pointers point to the right places and the ownership is handled propperly.
 *
 * The caller of this function MUST determine whether 'drain' or 'source'
 * maintains ownership.                                                       */
{
    TestAnalyzer_Buffer_assert_consistency(source);
    *drain = *source;
}

 bool 
TestAnalyzer_Buffer_has_intersection(TestAnalyzer_Buffer*       me,
                                   const TestAnalyzer_lexatom_t* Begin,
                                   ptrdiff_t                Size)
/* RETURNS: 'true' if buffer's memory intersects with the region given
 *                 by 'Begin' and 'Size'.
 *          'false', else.                                                    */
{
    const TestAnalyzer_lexatom_t* End      = &Begin[Size];
    TestAnalyzer_Buffer*       root     = TestAnalyzer_Buffer_nested_find_root(me);
    const TestAnalyzer_lexatom_t* my_begin = root->begin(root);
    const TestAnalyzer_lexatom_t* my_end   = me->end(me);

    /* No intersection in two cases:
     * (1) second interval lies completely before: 'End <= my_begin'.
     * (2) second interval lies completely after:  'Begin >= my_begin'.       */
    if( Begin >= my_end || End <= my_begin ) return false;
    else                                     return true;
}

 void
TestAnalyzer_Buffer_resources_absent_mark(TestAnalyzer_Buffer* me)
/* This function is to be called in case that construction failed. It marks
 * all resources as absent, such that destruction can handle it safely. 
 * Nevertheless, the member functions are put in place to be be sure that the
 * object is functional.                                                      */
{
    TestAnalyzer_Buffer_member_functions_assign(me);
    TestAnalyzer_Buffer_callbacks_set(me, (void (*)(void*))0,
                                         (void (*)(void*))0, (void*)0);
    me->filler = (TestAnalyzer_LexatomLoader*)0;
    TestAnalyzer_BufferMemory_resources_absent_mark(&me->_memory);
    TestAnalyzer_Buffer_member_functions_assert(me);
}

 bool    
TestAnalyzer_Buffer_resources_absent(TestAnalyzer_Buffer* me)
/* RETURNS: 'true' if all resources of buffer are absent. Then, nothing needs
 *                 to be freed.
 *          'false' else.                                                     */ 
{
    return    me->filler == (TestAnalyzer_LexatomLoader*)0 
           && TestAnalyzer_BufferMemory_resources_absent(&me->_memory);
}

 void     
TestAnalyzer_Buffer_dysfunctional_set(TestAnalyzer_Buffer*  me)
/* Set buffer into dysfunctional state, i.e. the buffer is inable to operarate.
 * Shall be applied only upon failure beyond repair.                          */
{
    TestAnalyzer_Buffer_init_analyzis_core(me, 
    /* ReadP                          */ (TestAnalyzer_lexatom_t*)0,
    /* LexatomStartP                  */ (TestAnalyzer_lexatom_t*)0,
    /* LexatomAtLexemeStart           */ (TestAnalyzer_lexatom_t)0,                                   
    /* LexatomBeforeLexemeStart       */ (TestAnalyzer_lexatom_t)0,
    /* BackupLexatomIndexOfReadP      */ (QUEX_TYPE_STREAM_POSITION)-1);
    TestAnalyzer_Buffer_member_functions_assert(me);
}

 bool     
TestAnalyzer_Buffer_dysfunctional(TestAnalyzer_Buffer*  me)
/* RETURNS: 'true' if buffer is in dysfunctional state.
 *          'false', else.                                                    */
{
    return    me->_read_p                         == (TestAnalyzer_lexatom_t*)0
           && me->_lexeme_start_p                 == (TestAnalyzer_lexatom_t*)0
           && me->_lexatom_at_lexeme_start        == (TestAnalyzer_lexatom_t)0                                   
           && me->_backup_lexatom_index_of_lexeme_start_p == (QUEX_TYPE_STREAM_POSITION)-1
            && me->_lexatom_before_lexeme_start == (TestAnalyzer_lexatom_t)0;
}

 void
TestAnalyzer_Buffer_init_analyzis(TestAnalyzer_Buffer*   me) 
/* Initialize:  _read_p                          
 *              _lexeme_start_p                 
 *              _lexatom_at_lexeme_start     
 *              _lexatom_before_lexeme_start                                  */
{
    if( ! me->_memory._front ) {
        /* No memory => FSM is put into a non-functional state.               */
        TestAnalyzer_Buffer_init_analyzis_core(me, 
        /* ReadP                          */ (TestAnalyzer_lexatom_t*)0,
        /* LexatomStartP                  */ (TestAnalyzer_lexatom_t*)0,
        /* LexatomAtLexemeStart           */ (TestAnalyzer_lexatom_t)0,                                   
        /* LexatomBeforeLexemeStart       */ (TestAnalyzer_lexatom_t)0,
        /* BackupLexatomIndexOfReadP      */ (QUEX_TYPE_STREAM_POSITION)-1);
    }
    else {
        /* The first state in the state machine does not increment. 
         * => input pointer is set to the first position, not before.         */
        TestAnalyzer_Buffer_init_analyzis_core(me, 
        /* ReadP                          */ me->content_begin(me),
        /* LexatomStartP                  */ me->content_begin(me),
        /* LexatomAtLexemeStart           */ (TestAnalyzer_lexatom_t)0,
        /* LexatomBeforeLexemeStart       */ QUEX_TestAnalyzer_SETTING_BUFFER_LEXATOM_NEWLINE,
        /* BackupLexatomIndexOfReadP      */ (QUEX_TYPE_STREAM_POSITION)-1);
    }
}

 void
TestAnalyzer_Buffer_init_analyzis_core(TestAnalyzer_Buffer*        me,
                                     TestAnalyzer_lexatom_t*        ReadP,
                                     TestAnalyzer_lexatom_t*        LexatomStartP,
                                     TestAnalyzer_lexatom_t         LexatomAtLexemeStart,
                                     TestAnalyzer_lexatom_t         LexatomBeforeLexemeStart,
                                     QUEX_TYPE_STREAM_POSITION BackupLexatomIndexOfReadP) 
{
    (void)LexatomBeforeLexemeStart;

    me->_read_p                  = ReadP;
    me->_lexeme_start_p          = LexatomStartP;
    me->_lexatom_at_lexeme_start = LexatomAtLexemeStart;                                   
    me->_fallback_n              = 0; /* To be set upon mode entry */
     me->_lexatom_before_lexeme_start    = LexatomBeforeLexemeStart;
    me->_backup_lexatom_index_of_lexeme_start_p = BackupLexatomIndexOfReadP;
}

 void
TestAnalyzer_Buffer_init_content(TestAnalyzer_Buffer* me, TestAnalyzer_lexatom_t* EndOfFileP)
/*  Initialize: input.lexatom_index_begin
 *              input.lexatom_index_end_of_stream                         
 *              input.end_p                                                   */
{
    TestAnalyzer_lexatom_t*        EndP             = me->content_space_end(me);
    QUEX_TYPE_STREAM_POSITION ci_begin         = (QUEX_TYPE_STREAM_POSITION)0;
    QUEX_TYPE_STREAM_POSITION ci_end_of_stream = (QUEX_TYPE_STREAM_POSITION)-1;
    TestAnalyzer_lexatom_t*        end_p            = (TestAnalyzer_lexatom_t*)0;
    (void)EndP;

    if( ! me->_memory._front ) {
        ci_end_of_stream = (QUEX_TYPE_STREAM_POSITION)-1;
        end_p            = (TestAnalyzer_lexatom_t*)0;
        ci_begin         = (QUEX_TYPE_STREAM_POSITION)-1;
    }
    else if( me->filler && me->filler->byte_loader ) {
        __quex_assert(0 == EndOfFileP);

        /* Setup condition to initiate immediate load when the state machine
         * is entered: 'read pointer hits buffer limit code'.                */
        ci_begin         = (QUEX_TYPE_STREAM_POSITION)0;
        ci_end_of_stream = (QUEX_TYPE_STREAM_POSITION)-1;
        end_p            = me->content_begin(me);
    } 
    else {
        __quex_assert(0 != me->_memory._front);           /* See first condition. */
        __quex_assert(! EndOfFileP || (EndOfFileP >= me->content_begin(me) && EndOfFileP <= EndP));

        if( EndOfFileP ) {
            ci_end_of_stream = EndOfFileP - me->content_begin(me);
            end_p            = EndOfFileP;   
        }
        else {
            ci_end_of_stream = (QUEX_TYPE_STREAM_POSITION)-1;
            end_p            = me->content_begin(me);   
        }
    }

    TestAnalyzer_Buffer_init_content_core(me, ci_begin, ci_end_of_stream, end_p);
}

 void
TestAnalyzer_Buffer_init_content_core(TestAnalyzer_Buffer* me, 
                                    QUEX_TYPE_STREAM_POSITION LI_Begin,
                                    QUEX_TYPE_STREAM_POSITION LI_EndOfStream,
                                    TestAnalyzer_lexatom_t*        EndOfFileP)
{
    me->input.lexatom_index_begin         = LI_Begin;
    me->input.lexatom_index_end_of_stream = LI_EndOfStream;
    if( EndOfFileP ) {
        me->input.end_p    = EndOfFileP;
        me->input.end_p[0] = QUEX_TestAnalyzer_SETTING_BUFFER_LEXATOM_BUFFER_BORDER;
        QUEX_IF_ASSERTS_poison(&me->content_end(me)[1], me->content_space_end(me));
    }
    else {
        me->input.end_p    = (TestAnalyzer_lexatom_t*)0;
    }
}

 void
TestAnalyzer_Buffer_register_content(TestAnalyzer_Buffer*        me,
                                   TestAnalyzer_lexatom_t*        EndOfInputP,
                                   QUEX_TYPE_STREAM_POSITION CharacterIndexBegin)
/* Registers information about the stream that fills the buffer and its
 * relation to the buffer. 
 *  
 *  EndOfInputP --> Position behind the last lexatom in the buffer that has
 *                  been streamed.
 *          '0' --> No change.
 *  
 *  CharacterIndexBegin --> Character index of the first lexatom in the 
 *                          buffer.
 *                 '-1' --> No change.                                       */
{
    if( EndOfInputP ) {
        __quex_assert(EndOfInputP <= me->content_space_end(me));
        __quex_assert(EndOfInputP >= me->content_begin(me));

        me->input.end_p    = EndOfInputP;
        me->input.end_p[0] = QUEX_TestAnalyzer_SETTING_BUFFER_LEXATOM_BUFFER_BORDER;
    }

    if( CharacterIndexBegin != (QUEX_TYPE_STREAM_POSITION)-1 ) {
        me->input.lexatom_index_begin = CharacterIndexBegin;
    }

    QUEX_IF_ASSERTS_poison(&me->content_end(me)[1], me->content_space_end(me));
    /* NOT: assert(TestAnalyzer_Buffer_input_lexatom_index_begin(me) >= 0);
     * This function may be called before content is setup/loaded propperly. */ 
}

 void       
TestAnalyzer_Buffer_register_eos(TestAnalyzer_Buffer*        me,
                               QUEX_TYPE_STREAM_POSITION CharacterIndexEndOfStream)
{
    me->input.lexatom_index_end_of_stream = CharacterIndexEndOfStream;
}

 bool
TestAnalyzer_Buffer_is_empty(TestAnalyzer_Buffer* me)
/* RETURNS: true, if buffer does not contain anything.
 *          false, else.                                                     */
{ 
    return    me->content_end(me) == me->content_begin(me) 
           && me->input.lexatom_index_begin == 0; 
}

 QUEX_TYPE_STREAM_POSITION  
TestAnalyzer_Buffer_input_lexatom_index_end(TestAnalyzer_Buffer* me)
/* RETURNS: Character index of the lexatom to which '.input.end_p' points.
 *                                                                           */
{
    __quex_assert(me->input.lexatom_index_begin >= 0);
    TestAnalyzer_Buffer_assert_pointers_in_range(me);

    return me->input.lexatom_index_begin + me->content_size(me);
}

 bool 
TestAnalyzer_Buffer_is_end_of_stream_inside(TestAnalyzer_Buffer* me)
{ 
    const ptrdiff_t ContentSpaceSize = me->content_space_size(me);

    if( me->input.lexatom_index_end_of_stream == (QUEX_TYPE_STREAM_POSITION)-1 ) {
        return false;
    }
    else if( me->input.lexatom_index_end_of_stream < me->input.lexatom_index_begin ) {
        return false;
    }
    else {
        return me->input.lexatom_index_end_of_stream - me->input.lexatom_index_begin < ContentSpaceSize;
    }
}

 bool 
TestAnalyzer_Buffer_is_end_of_stream(TestAnalyzer_Buffer* me)
{ 
    TestAnalyzer_Buffer_assert_consistency(me);
    if( me->input.lexatom_index_end_of_stream == (QUEX_TYPE_STREAM_POSITION)-1 ) {
        return false;
    }
    else if( me->_read_p != me->content_end(me) ) {
        return false;
    }

    return    TestAnalyzer_Buffer_input_lexatom_index_end(me) 
           == me->input.lexatom_index_end_of_stream;
}

 bool                  
TestAnalyzer_Buffer_is_begin_of_stream(TestAnalyzer_Buffer* buffer)
{ 
    TestAnalyzer_Buffer_assert_consistency(buffer);
    if     ( buffer->_lexeme_start_p != buffer->content_begin(buffer) ) return false;
    else if( TestAnalyzer_Buffer_input_lexatom_index_begin(buffer) )      return false;
    else                                                                return true;
}

 void
TestAnalyzer_Buffer_pointers_add_offset(TestAnalyzer_Buffer*  me,
                                      ptrdiff_t           offset,
                                      TestAnalyzer_lexatom_t** position_register,
                                      const size_t        PositionRegisterN)
/* Modify all buffer related pointers and lexatom indices by an offset. It is
 * assumed, that buffer content has been moved by 'offset' and that pointers
 * and offsets still related to the content before the move. 
 *
 * EXAMPLE: (lexatom index at buffer begin, read_p)
 *
 *                       0  1  2  3  4  5  6  7  8  9  10 11 12 13 14 15 
 * Stream:               [a][b][c][d][e][f][g][h][j][i][k][l][m][n][o][p]
 *
 * Buffer before move:           |[d][e][f][g][h]|
 *                                |        |
 *     lexatom_index_begin = 3 ---'        |
 *     read_p -----------------------------'
 *
 * Buffer after move, then ADD:  |[f][g][h][j][i]|
 *                                |   |
 *     lexatom_index_begin = 5 ---'   |
 *     read_p ------------------------'
 *
 * ADAPTS: _read_p, _lexeme_start_p, position registers, end_p, 
 *         input.end_lexatom_index                                            */
{ 
#   define QUEX_BUFFER_POINTER_ADD(P, BORDER, OFFSET)                \
           ((P) = (  ((P) == (TestAnalyzer_lexatom_t*)0) ? (P)            \
                   : ((BORDER) - (P) < OFFSET)     ? (BORDER)        \
                   : (P) + (OFFSET)))
#   define QUEX_BUFFER_POINTER_SUBTRACT(P, BORDER, NEGATIVE_OFFSET)  \
           ((P) = (  ((P) == (TestAnalyzer_lexatom_t*)0)     ? (P)        \
                   : ((BORDER) - (P) > NEGATIVE_OFFSET) ? (BORDER)   \
                   : (P) + (NEGATIVE_OFFSET)))
    TestAnalyzer_lexatom_t*  border = (TestAnalyzer_lexatom_t*)0;
    TestAnalyzer_lexatom_t** pit    = (TestAnalyzer_lexatom_t**)0x0;
    TestAnalyzer_lexatom_t** pEnd   = &position_register[PositionRegisterN];

    TestAnalyzer_Buffer_assert_pointers_in_range(me);

    if( offset > 0 ) {
        border = me->content_space_end(me);
        QUEX_BUFFER_POINTER_ADD(me->_read_p,         border, offset);
        QUEX_BUFFER_POINTER_ADD(me->_lexeme_start_p, border, offset);
        QUEX_BUFFER_POINTER_ADD(me->input.end_p,     border, offset);

        for(pit = position_register; pit != pEnd; ++pit) {
            QUEX_BUFFER_POINTER_ADD(*pit, border, offset); 
        }
    }
    else if( offset < 0 ) {
        border = me->content_begin(me);
        QUEX_BUFFER_POINTER_SUBTRACT(me->_read_p,         border, offset);
        QUEX_BUFFER_POINTER_SUBTRACT(me->_lexeme_start_p, border, offset);
        QUEX_BUFFER_POINTER_SUBTRACT(me->input.end_p,     border, offset);

        for(pit = position_register; pit != pEnd; ++pit) {
            QUEX_BUFFER_POINTER_SUBTRACT(*pit, border, offset); 
        }
    }
    else {
        return;
    }

    *(me->content_end(me)) = (TestAnalyzer_lexatom_t)QUEX_TestAnalyzer_SETTING_BUFFER_LEXATOM_BUFFER_BORDER;

    me->input.lexatom_index_begin -= offset;

    TestAnalyzer_Buffer_assert_pointers_in_range(me);

#   undef QUEX_BUFFER_POINTER_ADD
#   undef QUEX_BUFFER_POINTER_SUBTRACT
}

 TestAnalyzer_lexatom_t*
TestAnalyzer_Buffer_get_min_position_without_lexeme_start_p(TestAnalyzer_Buffer*  me, 
                                                          TestAnalyzer_lexatom_t** position_register, 
                                                          const size_t        PositionRegisterN)
/* Determines the earliest position in the buffer that needs to be maintained
 * for acceptance position / post context recovery. That is,
 *
 *         IT IS ASSUMED THAT THE LEXEME DOES NOT HAVE TO BE MAINTAINED!
 *_____________________________________________________________________________
 * NOTE: POSSIBLY, THE EXACT POSITION OF ACCEPTANCES OS OPTIMIZED AWAY, SO    *
 *       THAT IT MAY NOT BE STORED. THIS FUNCTION IS NOT SAFE AGAINST THAT.   *
 *_____________________________________________________________________________
 *
 * If an input position is set, it must lie in between the lexeme start pointer
 * and the current input pointer. Only those are considered. The 'smallest' one
 * is then returned.
 *
 * This function is useful for reload procedures in cases where there is no
 * need for the lexatom itself. As a result more data may be loaded into the
 * buffer.
 *
 * RETURNS: Min position in buffer that needs to be maintained in order to 
 *          recover position of last acceptance or begining of post context.  */
{
    TestAnalyzer_lexatom_t** pit              = (TestAnalyzer_lexatom_t**)0x0;
    TestAnalyzer_lexatom_t*  min_position     = (TestAnalyzer_lexatom_t*)0x0;
    TestAnalyzer_lexatom_t** Begin            = &position_register[0];
    TestAnalyzer_lexatom_t** End              = &position_register[PositionRegisterN];
    TestAnalyzer_lexatom_t*  relevant_begin_p = me->_lexeme_start_p ? me->_lexeme_start_p 
                                           : TestAnalyzer_Buffer_memory_content_begin(me);
    TestAnalyzer_lexatom_t*  relevant_end_p   = me->_read_p;
    TestAnalyzer_lexatom_t*  p                = (TestAnalyzer_lexatom_t*)0;

    TestAnalyzer_Buffer_assert_pointers_in_range(me);

    for(pit = Begin; pit != End; ++pit) {
        p = *pit - 1;
        if( p < relevant_begin_p || p >= relevant_end_p ) continue;
        if( min_position > p ) min_position = p;
    }
    return min_position ? min_position : relevant_end_p;
}

#if defined(QUEX_OPTION_DEBUG_SHOW_EXT)
 void
TestAnalyzer_Buffer_brief_trace_output(const TestAnalyzer_Buffer*  me, 
                                     TestAnalyzer_lexatom_t**       position_register, 
                                     const size_t              PositionRegisterN)
{
    /* lexatom_index --> position in the input stream
     * offset        --> offset from the buffer's first content lexatom       */
    const int buffer_lexatom_index_at_begin          = (int)(me->input.lexatom_index_begin);
    const int buffer_lexatom_index_at_end            = (int)(me->input.lexatom_index_begin + me->content_size(me));
    const int buffer_free_space_at_end_in_lexatoms   = (int)(me->content_space_size(me)    - me->content_size(me));
    const int lexeme_start_offset                    = (int)(me->_lexeme_start_p           - me->content_begin(me));
    const int lexeme_start_lexatom_index             = (int)(me->input.lexatom_index_begin + lexeme_start_offset);
    const int read_position_offset                   = (int)(me->_read_p                   - me->content_begin(me));
    const int read_position_lexatom_index            = (int)(me->input.lexatom_index_begin + read_position_offset);
    /* offset of 'end-of-stream' in buffer                                    */
    char      end_of_stream_offset[32];
    ptrdiff_t i, offset;

    if( me->input.lexatom_index_end_of_stream != -1 ) {
        __QUEX_STD_sprintf(end_of_stream_offset, "eos@%i", 
                           (int)(me->input.lexatom_index_end_of_stream - me->input.lexatom_index_begin));
    }
    else {
        __QUEX_STD_sprintf(end_of_stream_offset, "!eos");
    }

    __QUEX_STD_fprintf(QUEX_TestAnalyzer_SETTING_DEBUG_OUTPUT_CHANNEL, 
                       "b[%i-%i|+%i]:%s:l%i@%i:r%i@%i", 
                       buffer_lexatom_index_at_begin,
                       buffer_lexatom_index_at_end,
                       buffer_free_space_at_end_in_lexatoms,
                       end_of_stream_offset,
                       lexeme_start_offset,
                       lexeme_start_lexatom_index,
                       read_position_offset,
                       read_position_lexatom_index);

    QUEX_DEBUG_PRINT(":"); 
    if( PositionRegisterN == 0 ) {
        QUEX_DEBUG_PRINT("-:"); 
    }
    else {
        for(i=0;i<(ptrdiff_t)PositionRegisterN;++i) { 
            if( ! position_register[i] ) {
                QUEX_DEBUG_PRINT("?:"); 
            }
            else {
                QUEX_DEBUG_PRINT(i == 0 ? "a" : "pc");
                offset = position_register[i] - me->content_begin(me); 
                __QUEX_STD_fprintf(QUEX_TestAnalyzer_SETTING_DEBUG_OUTPUT_CHANNEL,
                                   "%i@%i:", 
                                   (int)(me->input.lexatom_index_begin + offset), (int)(offset));
            }
        }
    }
}
#endif



#include "test_c_cg/lib/buffer/lexatoms/LexatomLoader.i"
#include "test_c_cg/lib/buffer/Buffer_print.i"
#include "test_c_cg/lib/buffer/Buffer_navigation.i"
#include "test_c_cg/lib/buffer/Buffer_fill.i"
#include "test_c_cg/lib/buffer/Buffer_load.i"
#include "test_c_cg/lib/buffer/Buffer_nested.i"
#include "test_c_cg/lib/buffer/Buffer_callbacks.i"
#include "test_c_cg/lib/buffer/Buffer_invariance.i"
#include "test_c_cg/lib/buffer/Buffer_move.i"
#include "test_c_cg/lib/buffer/BufferMemory.i"

#endif /* QUEX_INCLUDE_GUARD_TestAnalyzer__BUFFER__BUFFER_I */
