/* -*- C++ -*- vim: set syntax=cpp: */
#ifndef QUEX_INCLUDE_GUARD_TestAnalyzer__BUFFER__ASSERTS_I
#define QUEX_INCLUDE_GUARD_TestAnalyzer__BUFFER__ASSERTS_I

#include "test_c_cg/lib/buffer/asserts"
#include "test_c_cg/lib/buffer/Buffer"

#if defined(QUEX_OPTION_ASSERTS)



 void
TestAnalyzer_BUFFER_ASSERT_INVARIANCE_SETUP(TestAnalyzer_BufferInvariance* bi, 
                                          TestAnalyzer_Buffer*           me)
{
    TestAnalyzer_BufferInvariance_construct(bi, me);
    TestAnalyzer_Buffer_assert_consistency(me);
}

 void
TestAnalyzer_BUFFER_ASSERT_INVARIANCE_VERIFY(TestAnalyzer_BufferInvariance* bi, 
                                           TestAnalyzer_Buffer*           me)     
{
    TestAnalyzer_Buffer_assert_consistency(me);                   
    TestAnalyzer_BufferInvariance_assert(bi, me, false);
}

 void
TestAnalyzer_BUFFER_ASSERT_INVARIANCE_VERIFY_SAME(TestAnalyzer_BufferInvariance* bi, 
                                                TestAnalyzer_Buffer*           me)     
{
    TestAnalyzer_Buffer_assert_consistency(me);                   
    TestAnalyzer_BufferInvariance_assert(bi, me, true);
}


 void
TestAnalyzer_Buffer_assert_pointers_in_range(const TestAnalyzer_Buffer* B)                                      
/* Check whether _read_p and _lexeme_start_p are in ther appropriate range. */
{                                                                                    
    __quex_assert( (B) != 0x0 );                                                     
    if( ! (*B)._memory._front && ! (*B)._memory._back ) {                    
        return;
    }

    __quex_assert((*B).begin(B)        <  (*B).content_space_end(B));                     
    __quex_assert((*B).content_end(B)  >= (*B).content_begin(B));          
    __quex_assert((*B).content_end(B)  <= (*B).content_space_end(B));               

    __quex_assert((*B)._read_p         >= (*B).begin(B));                
    __quex_assert((*B)._read_p         <= (*B).content_end(B));              
    __quex_assert((*B)._lexeme_start_p >= (*B).begin(B));                
    __quex_assert((*B)._lexeme_start_p <= (*B).content_end(B));              
}

 void
TestAnalyzer_Buffer_assert_limit_codes_in_place(const TestAnalyzer_Buffer* B)                                            
{
    if( ! (*B)._memory._front && ! (*B)._memory._back ) {                    
        return;
    }
    __quex_assert((*B).begin(B)[0]              == QUEX_TestAnalyzer_SETTING_BUFFER_LEXATOM_BUFFER_BORDER);    
    __quex_assert((*B).content_space_end(B)[0]  == QUEX_TestAnalyzer_SETTING_BUFFER_LEXATOM_BUFFER_BORDER);    
    __quex_assert((*B).content_end(B)[0]        == QUEX_TestAnalyzer_SETTING_BUFFER_LEXATOM_BUFFER_BORDER);   
}

 void
TestAnalyzer_Buffer_assert_consistency(const TestAnalyzer_Buffer* B)                                            
{                                                                                    
    const TestAnalyzer_Buffer* focus;
    if( ! B ) return;
    __quex_assert(   B->input.lexatom_index_begin == -1
                  || B->input.lexatom_index_begin >= 0);
    __quex_assert(   B->input.lexatom_index_end_of_stream == -1 
                  || B->input.lexatom_index_end_of_stream >= B->input.lexatom_index_begin);
    TestAnalyzer_Buffer_assert_pointers_in_range(B);                                              
    TestAnalyzer_Buffer_assert_limit_codes_in_place(B);

    if( B->_memory.ownership == E_Ownership_INCLUDING_BUFFER ) {
        __quex_assert(0 != B->_memory.including_buffer);
        /* No cyclic nesting of buffers.                                      */
        for(focus = B; 0 != focus->_memory.including_buffer; ) { 
            __quex_assert(focus->_memory.ownership == E_Ownership_INCLUDING_BUFFER);
            focus = focus->_memory.including_buffer;
            __quex_assert(focus != B);
        }
        /* NOT:
         *     __quex_assert(&including_buffer->_memory._back[1] == &front[0]);
         * BECAUSE: (1) Pointer adaption happens from back to front.
         *              => consistency could not be checked during adaption.
         *          (2) Future versions may store more in the buffer region.  */
    }
    else {
        __quex_assert(0 == B->_memory.including_buffer);
    }
    TestAnalyzer_Buffer_member_functions_assert(B);
}

 void
TestAnalyzer_Buffer_member_functions_assert(const TestAnalyzer_Buffer* me)
{
    __quex_assert(me->fill                == TestAnalyzer_Buffer_fill);
    __quex_assert(me->fill_prepare        == TestAnalyzer_Buffer_fill_prepare);
    __quex_assert(me->fill_finish         == TestAnalyzer_Buffer_fill_finish);

    __quex_assert(me->begin               == TestAnalyzer_Buffer_memory_begin);
    __quex_assert(me->end                 == TestAnalyzer_Buffer_memory_end);
    __quex_assert(me->size                == TestAnalyzer_Buffer_memory_size);

    __quex_assert(me->content_space_end   == TestAnalyzer_Buffer_memory_content_space_end);
    __quex_assert(me->content_space_size  == TestAnalyzer_Buffer_memory_content_space_size);

    __quex_assert(me->content_begin       == TestAnalyzer_Buffer_memory_content_begin);
    __quex_assert(me->content_end         == TestAnalyzer_Buffer_memory_content_end);
    __quex_assert(me->content_size        == TestAnalyzer_Buffer_memory_content_size);
}

 void
TestAnalyzer_Buffer_assert_no_lexatom_is_buffer_border(const TestAnalyzer_lexatom_t* Begin, 
                                                     const TestAnalyzer_lexatom_t* End)
{
    const TestAnalyzer_lexatom_t* iterator = 0x0;
    __quex_assert(Begin <= End);

    for(iterator = Begin; iterator != End; ++iterator) {
        if( *iterator != QUEX_TestAnalyzer_SETTING_BUFFER_LEXATOM_BUFFER_BORDER ) continue;

        if( iterator == Begin ) {
            QUEX_ERROR_EXIT("Buffer limit code character appeared as first character in buffer.\n"
                            "This is most probably a load failure.\n");
        } else {
            QUEX_ERROR_EXIT("Buffer limit code character appeared as normal text content.\n");
        }
    }
}



#endif

#endif /* QUEX_INCLUDE_GUARD_TestAnalyzer__BUFFER__ASSERTS_I */

