/* -*- C++ -*-   vim: set syntax=cpp:
 * (C) Frankt-Rene Schaefer
 * ABSOLUTELY NO WARRANTY               */
#ifndef QUEX_INCLUDE_GUARD_TestAnalyzer__ANALYZER__MEMBER__MISC_I
#define QUEX_INCLUDE_GUARD_TestAnalyzer__ANALYZER__MEMBER__MISC_I

#include "test_c_emm/lib/analyzer/Counter"
#include "test_c_emm/lib/buffer/Buffer_print"
#include "test_c_emm/lib/buffer/lexatoms/LexatomLoader"
#include "test_c_emm/lib/lexeme/basics"



 void 
TestAnalyzer_MF_error_code_clear(struct TestAnalyzer_tag* me)
{ me->error_code = E_Error_None; }

 void 
TestAnalyzer_MF_error_code_clear_this(struct TestAnalyzer_tag* me, E_Error ErrorCode)
{ if( me->error_code == ErrorCode ) me->error_code = E_Error_None; }

 void 
TestAnalyzer_MF_error_code_set_void(struct TestAnalyzer_tag* me)
{ me->error_code = E_Error_Uninitialized; }

 bool 
TestAnalyzer_MF_error_code_is_void(struct TestAnalyzer_tag* me)
{ return me->error_code == E_Error_Uninitialized; }

 void 
TestAnalyzer_MF_error_code_set_if_first(struct TestAnalyzer_tag* me, E_Error ErrorCode)
/* Never overwrite an error code
 * => original error is maintained.                                           */
{ if( me->error_code == E_Error_None ) me->error_code = ErrorCode; }

 struct TestAnalyzer_Token_tag*  
TestAnalyzer_MF_token_p(struct TestAnalyzer_tag* me)
{
    return me->_token_queue.write_iterator; 
}

 void 
TestAnalyzer_MF_send(struct TestAnalyzer_tag* me, 
                   TestAnalyzer_token_id_t  Id)
{ TestAnalyzer_TokenQueue_push(&me->_token_queue, Id); }

 void 
TestAnalyzer_MF_send_n(struct TestAnalyzer_tag* me, 
                     TestAnalyzer_token_id_t  Id, 
                     size_t              RepetitionN)
{ TestAnalyzer_TokenQueue_push_repeated(&me->_token_queue, Id, RepetitionN); }

 void 
TestAnalyzer_MF_send_text(struct TestAnalyzer_tag* me, 
                        TestAnalyzer_token_id_t  Id,
                        TestAnalyzer_lexatom_t*  BeginP, 
                        TestAnalyzer_lexatom_t*  EndP)
{ TestAnalyzer_TokenQueue_push_text(&me->_token_queue, Id, BeginP, EndP); }

 void 
TestAnalyzer_MF_send_string(struct TestAnalyzer_tag* me,
                          TestAnalyzer_token_id_t  Id,
                          TestAnalyzer_lexatom_t*  ZeroTerminatedString)
{ 
    const size_t Length = TestAnalyzer_lexeme_length((const TestAnalyzer_lexatom_t*)ZeroTerminatedString);
    TestAnalyzer_TokenQueue_push_text(&me->_token_queue, Id, ZeroTerminatedString, 
                                    ZeroTerminatedString + (ptrdiff_t)Length + 1); 
}

 bool
TestAnalyzer_MF_byte_order_reversion(struct TestAnalyzer_tag* me)
{ 
    __quex_assert(0 != me->buffer.filler);
    return me->buffer.filler->_byte_order_reversion_active_f; 
}

 void     
TestAnalyzer_MF_byte_order_reversion_set(struct TestAnalyzer_tag* me, bool Value)
{ 
    __quex_assert(0 != me->buffer.filler);
    me->buffer.filler->_byte_order_reversion_active_f = Value; 
}

 const char*
TestAnalyzer_MF_input_name(struct TestAnalyzer_tag* me)
{ return me->__input_name; }

 bool
TestAnalyzer_MF_input_name_set(struct TestAnalyzer_tag* me, const char* InputNameP)
/* Sets the 'input name', i.e. some string that identifies the input stream.
 * In case of failure '.__input_name' is set to NULL.
 *
 * RETURNS: true, for success. false, else.                                   */
{ 
    if( me->__input_name ) {
        quex_MemoryManager_free(me->__input_name, E_MemoryObjectType_INPUT_NAME);
    }
    if(  ! InputNameP ) {
        me->__input_name = (char*)0;
        return true;
    }
    else {
        me->__input_name = quex_MemoryManager_clone_string(InputNameP);
        return me->__input_name ? true : false;
    }
}

 const char* 
TestAnalyzer_MF_version(struct TestAnalyzer_tag* me)
{ 
    (void)me;
    return          QUEX_STRING(struct TestAnalyzer_tag)           \
           ": Version "         QUEX_TestAnalyzer_SETTING_ANALYZER_VERSION \
           ". Date "            QUEX_TestAnalyzer_SETTING_BUILD_DATE       \
           "Generated by Quex " QUEX_TestAnalyzer_SETTING_VERSION ".";
}

 void
TestAnalyzer_MF_print_this(struct TestAnalyzer_tag* me)
{
    QUEX_DEBUG_PRINT1("  mode:       %s;\n", 
                      me->__current_mode_p == 0x0 ? "0x0" : me->__current_mode_p->name);
    QUEX_DEBUG_PRINT1("  error_code: %s;\n", E_Error_NAME(me->error_code));

     TestAnalyzer_Counter_print_this(&me->counter);

    QUEX_DEBUG_PRINT("  _mode_stack: ");
    TestAnalyzer_ModeStack_print(&me->_mode_stack);

    TestAnalyzer_Buffer_print_this(&me->buffer);

    TestAnalyzer_user_print(me);

    if( me->error_code != E_Error_None ) {
        TestAnalyzer_Buffer_print_content_detailed(&me->buffer);
        QUEX_DEBUG_PRINT("\n\n");
    }
}


 size_t TestAnalyzer_MF_line_number(const struct TestAnalyzer_tag* me)            { return TestAnalyzer_MF_line_number_at_begin(me); }
 size_t TestAnalyzer_MF_line_number_at_begin(const struct TestAnalyzer_tag* me)   { return me->counter._line_number_at_begin; }
 size_t TestAnalyzer_MF_line_number_at_end(const struct TestAnalyzer_tag* me)     { return me->counter._line_number_at_end; }
 void   TestAnalyzer_MF_line_number_set(struct TestAnalyzer_tag* me, size_t Value) { me->counter._line_number_at_end = Value; }


 size_t TestAnalyzer_MF_column_number(const struct TestAnalyzer_tag* me)          { return TestAnalyzer_MF_column_number_at_begin(me); }
 size_t TestAnalyzer_MF_column_number_at_begin(const struct TestAnalyzer_tag* me) { return me->counter._column_number_at_begin; }
 size_t TestAnalyzer_MF_column_number_at_end(const struct TestAnalyzer_tag* me)   { return me->counter._column_number_at_end; }
 void   TestAnalyzer_MF_column_number_set(struct TestAnalyzer_tag* me, size_t Value) { me->counter._column_number_at_end = Value; }


 size_t  TestAnalyzer_MF_indentation(const struct TestAnalyzer_tag* me)           
{ return (size_t)(me->counter._indentation_stack.back - me->counter._indentation_stack.front) + (size_t)1; }






#endif /* QUEX_INCLUDE_GUARD_TestAnalyzer__ANALYZER__MEMBER__MISC_I */
