/* vim: set ft=c:
 * (C) Frank-Rene Schaefer */
#ifndef  QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_FILE_I
#define  QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_FILE_I

#include "test_c_emm/lib/quex/MemoryManager"
#include "test_c_emm/lib/quex/byte_loader/ByteLoader_FILE"



extern void                       quex_ByteLoader_FILE_chunk_seek(quex_ByteLoader* me, QUEX_TYPE_STREAM_POSITION Pos);
extern size_t                     quex_ByteLoader_FILE_chunk_load(quex_ByteLoader* me, void* buffer, const size_t ByteN, bool*);
extern void                       quex_ByteLoader_FILE_destruct(quex_ByteLoader* me);
extern void                       quex_ByteLoader_FILE_print_this(quex_ByteLoader* me);
extern bool                       quex_ByteLoader_FILE_compare_handle(const quex_ByteLoader* alter_ego_A, 
                                                                                 const quex_ByteLoader* alter_ego_B);



quex_ByteLoader*    
quex_ByteLoader_FILE_new(FILE* fh, bool BinaryModeF)
{
    quex_ByteLoader_FILE* me;
   
    me = (quex_ByteLoader_FILE*)quex_MemoryManager_allocate(sizeof(quex_ByteLoader_FILE),
                                                                     E_MemoryObjectType_BYTE_LOADER);
    if( ! me ) {
        return (quex_ByteLoader*)0;
    } 
    else if( ! quex_ByteLoader_FILE_construct(me, fh, BinaryModeF) ) {
        quex_MemoryManager_free(me, E_MemoryObjectType_BYTE_LOADER);
        return (quex_ByteLoader*)0;
    }
    else {
        (me->base).ownership = E_Ownership_LEXER;
        return &(me->base);
    }
}

quex_ByteLoader*    
quex_ByteLoader_FILE_new_from_file_name(const char* FileName)
{
    quex_ByteLoader_FILE* me;

    me = (quex_ByteLoader_FILE*)quex_MemoryManager_allocate(sizeof(quex_ByteLoader_FILE),
                                                                     E_MemoryObjectType_BYTE_LOADER);
    if( ! me ) {
        return (quex_ByteLoader*)0;
    }
    else if( ! quex_ByteLoader_FILE_construct_from_file_name(me, FileName) ) {
        quex_MemoryManager_free(me, E_MemoryObjectType_BYTE_LOADER);
        return (quex_ByteLoader*)0;
    }
    else {
        (me->base).ownership = E_Ownership_LEXER;
        return &(me->base);
    }
}

bool
quex_ByteLoader_FILE_construct_from_file_name(quex_ByteLoader_FILE* me, 
                                                        const char*                      FileName)
{
    FILE*  fh = fopen(FileName, "rb");

    return quex_ByteLoader_FILE_construct(me, fh, true);
}

bool
quex_ByteLoader_FILE_construct(quex_ByteLoader_FILE* me, FILE* fh, bool BinaryModeF)
{
    /* IMPORTANT: input_handle must be set BEFORE call to base constructor!
     *            Constructor does call 'tell()'                             */
    QUEX_TYPE_STREAM_POSITION chunk_position_zero = fh ? (QUEX_TYPE_STREAM_POSITION )ftell(fh) 
                                                       : (QUEX_TYPE_STREAM_POSITION )0;
    me->input_handle = fh;

    quex_ByteLoader_construct(&(me->base), BinaryModeF,
                                         /* ChunkSizeInBytes */ 1,
                                         /* ChunkPositionOfReadByteIdxZero */ chunk_position_zero,
                                         quex_ByteLoader_FILE_chunk_seek,
                                         quex_ByteLoader_FILE_chunk_load,
                                         quex_ByteLoader_FILE_destruct,
                                         quex_ByteLoader_FILE_print_this,
                                         quex_ByteLoader_FILE_compare_handle);
    return fh ? true : false;
}

void    
quex_ByteLoader_FILE_destruct(quex_ByteLoader* alter_ego)
{
    quex_ByteLoader_FILE* me = (quex_ByteLoader_FILE*)(alter_ego);

    if( me->input_handle ) {
        fclose(me->input_handle);
        me->input_handle = (FILE*)0;
    }
}

void                      
quex_ByteLoader_FILE_chunk_seek(quex_ByteLoader* alter_ego, QUEX_TYPE_STREAM_POSITION Pos) 
{ 
    quex_ByteLoader_FILE* me = (quex_ByteLoader_FILE*)(alter_ego);
    if( ! me->input_handle ) return;
    (void)fseek(me->input_handle, (long)Pos, SEEK_SET); 
}

size_t  
quex_ByteLoader_FILE_chunk_load(quex_ByteLoader* alter_ego, 
                                void*                  buffer, 
                                const size_t           ByteN, 
                                bool*                  end_of_stream_f) 
{ 
    quex_ByteLoader_FILE* me = (quex_ByteLoader_FILE*)(alter_ego);
    size_t                      loaded_byte_n;

    if( ! me->input_handle ) { *end_of_stream_f = true; return 0; }

    loaded_byte_n = fread(buffer, 1, ByteN, me->input_handle); 
    *end_of_stream_f = feof(me->input_handle) ? true : false;
    return loaded_byte_n;
}

bool  
quex_ByteLoader_FILE_compare_handle(const quex_ByteLoader* alter_ego_A, 
                                          const quex_ByteLoader* alter_ego_B) 
/* RETURNS: true  -- if A and B point to the same FILE object.
 *          false -- else.                                                   */
{ 
    const quex_ByteLoader_FILE* A = (quex_ByteLoader_FILE*)(alter_ego_A);
    const quex_ByteLoader_FILE* B = (quex_ByteLoader_FILE*)(alter_ego_B);

    return A->input_handle == B->input_handle;
}

void                       
quex_ByteLoader_FILE_print_this(quex_ByteLoader* alter_ego)
{
    quex_ByteLoader_FILE* me = (quex_ByteLoader_FILE*)(alter_ego);

    QUEX_DEBUG_PRINT("        type:             FILE;\n");
    QUEX_DEBUG_PRINT1("        file_handle:      ((%p));\n", (const void*)me->input_handle);
    if( me->input_handle ) {
        QUEX_DEBUG_PRINT1("        end_of_stream:    %s;\n", E_Boolean_NAME(feof(me->input_handle)));
    }
}



#endif /*  QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_FILE_I */
