/* vim: set ft=c:
 * (C) Frank-Rene Schaefer */
#ifndef  QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_MEMORY_I
#define  QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_MEMORY_I

#include "test_c_emm/lib/quex/MemoryManager"
#include "test_c_emm/lib/quex/byte_loader/ByteLoader_Memory"
#include "test_c_emm/lib/quex/asserts"
#include <malloc.h> // DEBUG



extern void                       quex_ByteLoader_Memory_chunk_seek(quex_ByteLoader* me, 
                                                                        QUEX_TYPE_STREAM_POSITION   Pos);
extern size_t                     quex_ByteLoader_Memory_chunk_load(quex_ByteLoader* me, 
                                                                        void*                       buffer, 
                                                                        const size_t                ByteN, 
                                                                        bool*                       end_of_stream_f);
extern void                       quex_ByteLoader_Memory_destruct(quex_ByteLoader* me);
extern void                       quex_ByteLoader_Memory_print_this(quex_ByteLoader* me);
extern bool                       quex_ByteLoader_Memory_compare_handle(const quex_ByteLoader* alter_ego_A, 
                                                                                   const quex_ByteLoader* alter_ego_B);



quex_ByteLoader*    
quex_ByteLoader_Memory_new(const uint8_t*  BeginP,
                                     const uint8_t*  EndP)
{
    quex_ByteLoader_Memory* me;
   
    me = (quex_ByteLoader_Memory*)quex_MemoryManager_allocate(sizeof(quex_ByteLoader_Memory),
                                                                                    E_MemoryObjectType_BYTE_LOADER);
    if( ! me ) {
        return (quex_ByteLoader*)0;
    }
    else if( ! quex_ByteLoader_Memory_construct(me, BeginP, EndP) ) {
        quex_MemoryManager_free(me, E_MemoryObjectType_BYTE_LOADER);
        return (quex_ByteLoader*)0;
    }
    else {
        (me->base).ownership = E_Ownership_LEXER;
        return &(me->base);
    }
}

quex_ByteLoader*    
quex_ByteLoader_Memory_new_from_file_name(const char* FileName)
{
    quex_ByteLoader_Memory* me;
   
    me = (quex_ByteLoader_Memory*)quex_MemoryManager_allocate(sizeof(quex_ByteLoader_Memory),
                                                                                    E_MemoryObjectType_BYTE_LOADER);
    if( ! me ) {
        return (quex_ByteLoader*)0;
    }
    else if( ! quex_ByteLoader_Memory_construct_from_file_name(me, FileName) ) {
        quex_MemoryManager_free(me, E_MemoryObjectType_BYTE_LOADER);
        return (quex_ByteLoader*)0;
    }
    else {
        (me->base).ownership = E_Ownership_LEXER;
        return &(me->base);
    }
}

bool
quex_ByteLoader_Memory_construct_from_file_name(quex_ByteLoader_Memory* me, 
                                                          const char*                        FileName)
{
    size_t                  size;
    uint8_t*                begin_p;

    /* Determine size of file                                                */;
    FILE* fh = fopen(FileName, "rb"); 
    if( ! fh ) {
        return false;
    }
    if( fseek(fh, 0, SEEK_END) ) {
        fclose(fh);
        return false;
    }
    size = (size_t)ftell(fh);
    if( fseek(fh, 0, SEEK_SET) ) {
        fclose(fh);
        return false;
    }

    /* Load the file's content into some memory.                             */
    begin_p = (uint8_t*)quex_MemoryManager_allocate(size, E_MemoryObjectType_BUFFER_MEMORY);
    if( size != fread(begin_p, 1, size, fh) ) {
        quex_MemoryManager_free(begin_p, E_MemoryObjectType_BUFFER_MEMORY);
        fclose(fh);
        return false;
    }
    else if( ! quex_ByteLoader_Memory_construct(me, begin_p, &begin_p[size])) {
        quex_MemoryManager_free(begin_p, E_MemoryObjectType_BUFFER_MEMORY);
        fclose(fh);
        return false;
    }
    else {
        /* Mark memory ownership => destructor deletes it.                   */
        ((quex_ByteLoader_Memory*)me)->memory_ownership = E_Ownership_LEXER;
        fclose(fh);
        return true;
    }
}

bool
quex_ByteLoader_Memory_construct(quex_ByteLoader_Memory* me, 
                                           const uint8_t*                BeginP,
                                           const uint8_t*                EndP)
{
    __quex_assert(EndP >= BeginP);

    me->byte_array.begin_p  = BeginP;
    me->byte_array.position = BeginP;
    me->byte_array.end_p    = EndP;
    quex_ByteLoader_construct(&(me->base), true,
                                         /* ChunkSizeInBytes */1,
                                         /* ChunkPositionOfReadByteIdxZero */0,
                                         quex_ByteLoader_Memory_chunk_seek,
                                         quex_ByteLoader_Memory_chunk_load,
                                         quex_ByteLoader_Memory_destruct,
                                         quex_ByteLoader_Memory_print_this,
                                         quex_ByteLoader_Memory_compare_handle);

    return (! BeginP || ! EndP || BeginP > EndP) ? false : true;
}

void    
quex_ByteLoader_Memory_destruct(quex_ByteLoader* alter_ego)
{
    /* NOTE: The momory's ownership remains in the hand of the one who
     *       constructed this object.                                        */
    quex_ByteLoader_Memory* me = (quex_ByteLoader_Memory*)(alter_ego);

    if( me->memory_ownership == E_Ownership_LEXER ) {
        quex_MemoryManager_free((void*)&me->byte_array.begin_p[0], 
                                           E_MemoryObjectType_BUFFER_MEMORY);
    }
}

void                      
quex_ByteLoader_Memory_chunk_seek(quex_ByteLoader* alter_ego, QUEX_TYPE_STREAM_POSITION Pos) 
{ 
    quex_ByteLoader_Memory* me = (quex_ByteLoader_Memory*)(alter_ego);

    if( Pos > me->byte_array.end_p -  me->byte_array.begin_p ) {
        /* Make sure, that the 'load()' will not provide data!                */
        me->byte_array.position = &me->byte_array.end_p[0];
    }
    else {
        me->byte_array.position = &me->byte_array.begin_p[(ptrdiff_t)Pos];
    }
}

size_t  
quex_ByteLoader_Memory_chunk_load(quex_ByteLoader* alter_ego, 
                                void*                    buffer, 
                                const size_t             ByteN, 
                                bool*                    end_of_stream_f) 
{ 
    quex_ByteLoader_Memory* me = (quex_ByteLoader_Memory*)(alter_ego);
    const ptrdiff_t               Remaining = me->byte_array.end_p - me->byte_array.position;
    ptrdiff_t                     copy_n;

    if( (size_t)Remaining < ByteN ) { copy_n = Remaining; *end_of_stream_f = true; }
    else                            { copy_n = ByteN; } 

    memcpy((void*)buffer, (void*)me->byte_array.position, copy_n);
    me->byte_array.position += copy_n;
    return copy_n;
}

bool  
quex_ByteLoader_Memory_compare_handle(const quex_ByteLoader* alter_ego_A, 
                                            const quex_ByteLoader* alter_ego_B) 
/* RETURNS: true  -- if A and B point to the same Memory object.
 *          false -- else.                                                   */
{ 
    const quex_ByteLoader_Memory* A = (quex_ByteLoader_Memory*)(alter_ego_A);
    const quex_ByteLoader_Memory* B = (quex_ByteLoader_Memory*)(alter_ego_B);

    return    A->byte_array.begin_p  == B->byte_array.begin_p
           && A->byte_array.end_p    == B->byte_array.end_p
           && A->byte_array.position == B->byte_array.position;
}

void                       
quex_ByteLoader_Memory_print_this(quex_ByteLoader* alter_ego)
{
    quex_ByteLoader_Memory* me = (quex_ByteLoader_Memory*)(alter_ego);

    QUEX_DEBUG_PRINT("        type:             memory;\n");
    QUEX_DEBUG_PRINT1("        memory_ownership: %s;\n", E_Ownership_NAME(me->memory_ownership));
    QUEX_DEBUG_PRINT3("        byte_array:       { begin: ((%p)) end: ((%p)) size: %i; }\n",
                      (const void*)me->byte_array.begin_p, 
                      (const void*)me->byte_array.end_p, 
                      (int)(me->byte_array.end_p - me->byte_array.begin_p));
    QUEX_DEBUG_PRINT("        input_position:   ");
    quex_print_relative_positions(me->byte_array.begin_p, me->byte_array.end_p, 1, 
                                     (void*)me->byte_array.position);
    QUEX_DEBUG_PRINT("\n");
}



#endif /* QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_MEMORY_I */
