/* vim: set ft=c:
 * (C) Frank-Rene Schaefer */
#ifndef  QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_MONITOR_I
#define  QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_MONITOR_I

#include "test_c_emm/lib/quex/MemoryManager"
#include "test_c_emm/lib/quex/byte_loader/ByteLoader_Monitor"
#include "quex/byte_loader/ByteLoader_Monitor"
#include <malloc.h> // DEBUG



extern void                       quex_ByteLoader_Monitor_construct(quex_ByteLoader_Monitor* me,
                                                                            quex_ByteLoader*       source,
                                                                            void*                        reference_object);
extern void                       quex_ByteLoader_Monitor_chunk_seek(quex_ByteLoader*     me, 
                                                                       QUEX_TYPE_STREAM_POSITION  Pos);
extern size_t                     quex_ByteLoader_Monitor_chunk_load(quex_ByteLoader* me, 
                                                                       void*                  buffer, 
                                                                       const size_t           ByteN, 
                                                                       bool*                  end_of_stream_f);
extern void                       quex_ByteLoader_Monitor_destruct(quex_ByteLoader* me);
extern void                       quex_ByteLoader_Monitor_print_this(quex_ByteLoader* me);
extern bool                       quex_ByteLoader_Monitor_compare_handle(const quex_ByteLoader* alter_ego_A, 
                                                                                  const quex_ByteLoader* alter_ego_B);



quex_ByteLoader*    
quex_ByteLoader_Monitor_new(quex_ByteLoader* source,
                                    void*                       reference_object)
    /* ByteLoader takes over ownership over 'source' */
{
    quex_ByteLoader_Monitor* me;
   
    me = (quex_ByteLoader_Monitor*)quex_MemoryManager_allocate(
                   sizeof(quex_ByteLoader_Monitor),
                   E_MemoryObjectType_BYTE_LOADER);

    if( ! me ) return (quex_ByteLoader*)0;

    quex_ByteLoader_Monitor_construct(me, source, reference_object);

    return &(me->base);
}

void
quex_ByteLoader_Monitor_construct(quex_ByteLoader_Monitor* me, 
                                          quex_ByteLoader*       source,
                                          void*                        reference_object)
{
    me->source           = source;
    me->reference_object = reference_object;
    QUEX_TYPE_STREAM_POSITION chunk_size_in_bytes = source ? source->chunk_size_in_bytes : 0;
    QUEX_TYPE_STREAM_POSITION chunk_position_zero = source ? source->chunk_position_of_read_byte_i_zero : 0;
 
    quex_ByteLoader_construct(&(me->base), source->binary_mode_f,
                                         chunk_size_in_bytes,
                                         chunk_position_of_read_byte_i_zero,
                                         quex_ByteLoader_Monitor_chunk_seek,
                                         quex_ByteLoader_Monitor_chunk_load,
                                         quex_ByteLoader_Monitor_destruct,
                                         quex_ByteLoader_Monitor_print_this,
                                         quex_ByteLoader_Monitor_compare_handle);
}

void    
quex_ByteLoader_Monitor_destruct(quex_ByteLoader* alter_ego)
{
    quex_ByteLoader_Monitor* me = (quex_ByteLoader_Monitor*)(alter_ego);

    if( me->on_destruct ) {
        me->on_destruct(me);
    }
    me->source->destruct(me->source);
    quex_ByteLoader_delete(me->source);
}

void                      
quex_ByteLoader_Monitor_chunk_seek(quex_ByteLoader* alter_ego, QUEX_TYPE_STREAM_POSITION Pos) 
{ 
    quex_ByteLoader_Monitor* me = (quex_ByteLoader_Monitor*)(alter_ego);
    QUEX_TYPE_STREAM_POSITION    position;
    QUEX_TYPE_STREAM_POSITION    result_position;

    if( me->on_seek ) {
        position = me->on_seek(me, Pos);
    }
    else {
        position = Pos;
    }

    me->source->seek(me->source, position);

    ++(me->seek_n);
    me->position_last_seek = position;
}

size_t  
quex_ByteLoader_Monitor_chunk_load(quex_ByteLoader*   alter_ego, 
                                 void*                    buffer, 
                                 const size_t             ByteN, 
                                 bool*                    end_of_stream_f) 
{ 
    quex_ByteLoader_Monitor* me = (quex_ByteLoader_Monitor*)(alter_ego);
    size_t                       loaded_byte_n;
    size_t                       byte_n;

    if( me->on_before_load ) {
        byte_n = me->on_before_load(me, ByteN);
    }
    else {
        byte_n = ByteN;
    }

    loaded_byte_n = me->source->load(me->source, buffer, byte_n, end_of_stream_f);

    if( me->on_after_load ) {
        loaded_byte_n = me->on_after_load(me, buffer, loaded_byte_n, end_of_stream_f);
    }

    ++(me->load_n);
    me->loaded_byte_n += loaded_byte_n;

    return loaded_byte_n;
}

bool  
quex_ByteLoader_Monitor_compare_handle(const quex_ByteLoader* alter_ego_A, 
                                                      const quex_ByteLoader* alter_ego_B) 
/* RETURNS: true  -- if A and B point to the same Memory object.
 *          false -- else.                                                   */
{ 
    const quex_ByteLoader_Monitor* me = (quex_ByteLoader_Monitor*)(alter_ego_A);
    bool                                          result;

    result = me->source->compare_handle(me->source, alter_ego_B);

    return result;
}

void                       
quex_ByteLoader_Monitor_print_this(quex_ByteLoader* alter_ego)
{
    quex_ByteLoader_Monitor* me = (quex_ByteLoader_Monitor*)(alter_ego);

    QUEX_DEBUG_PRINT("        remote_controlled: {\n");
    me->source->print_this(me->source);
    QUEX_DEBUG_PRINT("        }\n");
}



#endif /* QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_MONITOR_I */
