#ifndef QUEX_INCLUDE_GUARD_TestAnalyzer__ANALYZER__COUNTER_I
#define QUEX_INCLUDE_GUARD_TestAnalyzer__ANALYZER__COUNTER_I

#include "test_cpp/lib/definitions"
#include "test_cpp/lib/analyzer/asserts"
#include "test_cpp/lib/analyzer/Counter"



inline bool
TestAnalyzer_Counter_construct(TestAnalyzer_Counter* me)
{
       me->_line_number_at_begin   = (size_t)1;
       me->_line_number_at_end     = (size_t)1;
     me->_column_number_at_begin = (size_t)1;
     me->_column_number_at_end   = (size_t)1;

      TestAnalyzer_IndentationStack_init(&me->_indentation_stack);
    return true;
}

inline void
TestAnalyzer_Counter_resources_absent_mark(TestAnalyzer_Counter* me)
{
     me->_line_number_at_begin               = (size_t)0;
     me->_line_number_at_end                 = (size_t)0;
     me->_column_number_at_begin           = (size_t)0;
     me->_column_number_at_end             = (size_t)0;
     me->_indentation_stack.back       = (TestAnalyzer_indentation_t*)0;
     me->_indentation_stack.memory_end = (TestAnalyzer_indentation_t*)0;
}

inline bool
TestAnalyzer_Counter_resources_absent(TestAnalyzer_Counter* me)
{
     if( me->_line_number_at_begin               != (size_t)0) return false;
     if( me->_line_number_at_end                 != (size_t)0) return false;
     if( me->_column_number_at_begin           != (size_t)0) return false;
     if( me->_column_number_at_end             != (size_t)0) return false;
     if( me->_indentation_stack.back       != (TestAnalyzer_indentation_t*)0) return false;
     if( me->_indentation_stack.memory_end != (TestAnalyzer_indentation_t*)0) return false;
    return true;
}


inline void 
TestAnalyzer_Counter_print_this(TestAnalyzer_Counter* me)
{
     TestAnalyzer_indentation_t* it = 0x0;

    QUEX_DEBUG_PRINT("  counter: ");
    if( TestAnalyzer_Counter_resources_absent(me) ) {
        QUEX_DEBUG_PRINT("<unitialized>\n");
        return;
    }
    QUEX_DEBUG_PRINT("{\n");


    QUEX_DEBUG_PRINT1("    _line_number_at_begin:   %i;\n", (int)me->_line_number_at_begin);
    QUEX_DEBUG_PRINT1("    _line_number_at_end:     %i;\n", (int)me->_line_number_at_end);


    QUEX_DEBUG_PRINT1("    _column_number_at_begin: %i;\n", (int)me->_column_number_at_begin);
    QUEX_DEBUG_PRINT1("    _column_number_at_end:   %i;\n", (int)me->_column_number_at_end);


    QUEX_DEBUG_PRINT("    _indentation_stack: [");
    for(it  = me->_indentation_stack.front; 
           it != &me->_indentation_stack.back[1] 
        && it != &me->_indentation_stack.front[QUEX_TestAnalyzer_SETTING_INDENTATION_STACK_SIZE]; 
        ++it) {
        QUEX_DEBUG_PRINT1("%i, ", (int)*it);
    }
    QUEX_DEBUG_PRINT("]\n");

    QUEX_DEBUG_PRINT("  }\n");
}



inline void
TestAnalyzer_IndentationStack_init(TestAnalyzer_IndentationStack* me)
{
    *(me->front)   = 1;          /* first indentation at column = 1 */
    me->back       = me->front;
    me->memory_end = &me->front[QUEX_TestAnalyzer_SETTING_INDENTATION_STACK_SIZE];
}


#ifdef QUEX_OPTION_ASSERTS
inline void
TestAnalyzer_Counter_assert_consistency(TestAnalyzer_Counter* me)
{

    /* The line number can never decrease.                                   */ 
    __quex_assert(me->_line_number_at_begin <= me->_line_number_at_end);            
    /* Line number remained the same => column number *must* have increased. */ 
    /* There is not pattern of a length less than 1                          */ 
    if(me->_line_number_at_begin == me->_line_number_at_end ) {                     
        __quex_assert(me->_column_number_at_begin < me->_column_number_at_end);     
    }                                                                                   


    /* If only column numbers are counted, then no assumptions can be made 
     * about increase or decrease. If a newline appears, for example, the 
     * column number may decrease.                                            */
}
#endif



#endif /* QUEX_INCLUDE_GUARD_TestAnalyzer__ANALYZER__COUNTER_I */

