/* vim: set ft=c:
 * (C) Frank-Rene Schaefer */
#ifndef  QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_MONITOR_I
#define  QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_MONITOR_I

#include "test_cpp/lib/quex/MemoryManager"
#include "test_cpp/lib/quex/byte_loader/ByteLoader_Monitor"
#include "quex/byte_loader/ByteLoader_Monitor"
#include <malloc.h> // DEBUG

namespace quex {

extern void                       ByteLoader_Monitor_construct(quex::ByteLoader_Monitor* me,
                                                                            quex::ByteLoader*       source,
                                                                            void*                        reference_object);
extern void                       ByteLoader_Monitor_chunk_seek(quex::ByteLoader*     me, 
                                                                       QUEX_TYPE_STREAM_POSITION  Pos);
extern size_t                     ByteLoader_Monitor_chunk_load(quex::ByteLoader* me, 
                                                                       void*                  buffer, 
                                                                       const size_t           ByteN, 
                                                                       bool*                  end_of_stream_f);
extern void                       ByteLoader_Monitor_destruct(quex::ByteLoader* me);
extern void                       ByteLoader_Monitor_print_this(quex::ByteLoader* me);
extern bool                       ByteLoader_Monitor_compare_handle(const quex::ByteLoader* alter_ego_A, 
                                                                                  const quex::ByteLoader* alter_ego_B);


ByteLoader_Monitor::ByteLoader_Monitor(quex::ByteLoader* source,
                                                                 void*                       reference_object)
{
    ByteLoader_Monitor_construct(this, source, reference_object);
}


quex::ByteLoader*    
ByteLoader_Monitor_new(quex::ByteLoader* source,
                                    void*                       reference_object)
    /* ByteLoader takes over ownership over 'source' */
{
    quex::ByteLoader_Monitor* me;
   
    me = (quex::ByteLoader_Monitor*)quex::MemoryManager_allocate(
                   sizeof(quex::ByteLoader_Monitor),
                   E_MemoryObjectType_BYTE_LOADER);

    if( ! me ) return (quex::ByteLoader*)0;

    quex::ByteLoader_Monitor_construct(me, source, reference_object);

    return &(*me);
}

void
ByteLoader_Monitor_construct(quex::ByteLoader_Monitor* me, 
                                          quex::ByteLoader*       source,
                                          void*                        reference_object)
{
    me->source           = source;
    me->reference_object = reference_object;
    QUEX_TYPE_STREAM_POSITION chunk_size_in_bytes = source ? source->chunk_size_in_bytes : 0;
    QUEX_TYPE_STREAM_POSITION chunk_position_zero = source ? source->chunk_position_of_read_byte_i_zero : 0;
 
    quex::ByteLoader_construct(&(*me), source->binary_mode_f,
                                         chunk_size_in_bytes,
                                         chunk_position_of_read_byte_i_zero,
                                         quex::ByteLoader_Monitor_chunk_seek,
                                         quex::ByteLoader_Monitor_chunk_load,
                                         quex::ByteLoader_Monitor_destruct,
                                         quex::ByteLoader_Monitor_print_this,
                                         quex::ByteLoader_Monitor_compare_handle);
}

void    
ByteLoader_Monitor_destruct(quex::ByteLoader* alter_ego)
{
    quex::ByteLoader_Monitor* me = (quex::ByteLoader_Monitor*)(alter_ego);

    if( me->on_destruct ) {
        me->on_destruct(me);
    }
    me->source->destruct(me->source);
    quex::ByteLoader_delete(me->source);
}

void                      
ByteLoader_Monitor_chunk_seek(quex::ByteLoader* alter_ego, QUEX_TYPE_STREAM_POSITION Pos) 
{ 
    quex::ByteLoader_Monitor* me = (quex::ByteLoader_Monitor*)(alter_ego);
    QUEX_TYPE_STREAM_POSITION    position;
    QUEX_TYPE_STREAM_POSITION    result_position;

    if( me->on_seek ) {
        position = me->on_seek(me, Pos);
    }
    else {
        position = Pos;
    }

    me->source->seek(me->source, position);

    ++(me->seek_n);
    me->position_last_seek = position;
}

size_t  
ByteLoader_Monitor_chunk_load(quex::ByteLoader*   alter_ego, 
                                 void*                    buffer, 
                                 const size_t             ByteN, 
                                 bool*                    end_of_stream_f) 
{ 
    quex::ByteLoader_Monitor* me = (quex::ByteLoader_Monitor*)(alter_ego);
    size_t                       loaded_byte_n;
    size_t                       byte_n;

    if( me->on_before_load ) {
        byte_n = me->on_before_load(me, ByteN);
    }
    else {
        byte_n = ByteN;
    }

    loaded_byte_n = me->source->load(me->source, buffer, byte_n, end_of_stream_f);

    if( me->on_after_load ) {
        loaded_byte_n = me->on_after_load(me, buffer, loaded_byte_n, end_of_stream_f);
    }

    ++(me->load_n);
    me->loaded_byte_n += loaded_byte_n;

    return loaded_byte_n;
}

bool  
ByteLoader_Monitor_compare_handle(const quex::ByteLoader* alter_ego_A, 
                                                      const quex::ByteLoader* alter_ego_B) 
/* RETURNS: true  -- if A and B point to the same Memory object.
 *          false -- else.                                                   */
{ 
    const quex::ByteLoader_Monitor* me = (quex::ByteLoader_Monitor*)(alter_ego_A);
    bool                                          result;

    result = me->source->compare_handle(me->source, alter_ego_B);

    return result;
}

void                       
ByteLoader_Monitor_print_this(quex::ByteLoader* alter_ego)
{
    quex::ByteLoader_Monitor* me = (quex::ByteLoader_Monitor*)(alter_ego);

    QUEX_DEBUG_PRINT("        remote_controlled: {\n");
    me->source->print_this(me->source);
    QUEX_DEBUG_PRINT("        }\n");
}

} /* close quex */

#endif /* QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_MONITOR_I */
