/* vim: ft=c:
 * (C) Frank-Rene Schaefer */
#ifndef  QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_POSIX_I
#define  QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_POSIX_I

#include "test_cpp/lib/quex/MemoryManager"
#include "test_cpp/lib/quex/byte_loader/ByteLoader_POSIX"

namespace quex {

extern void                       ByteLoader_POSIX_chunk_seek(quex::ByteLoader* me, 
                                                                       QUEX_TYPE_STREAM_POSITION Pos);
extern size_t                     ByteLoader_POSIX_chunk_load(quex::ByteLoader* me, 
                                                                       void* buffer, const size_t ByteN, 
                                                                       bool*);
extern void                       ByteLoader_POSIX_destruct(quex::ByteLoader* me);
extern void                       ByteLoader_POSIX_print_this(quex::ByteLoader* me);
extern bool                       ByteLoader_POSIX_compare_handle(const quex::ByteLoader* alter_ego_A, 
                                                                       const quex::ByteLoader* alter_ego_B);


ByteLoader_POSIX::ByteLoader_POSIX(int fd)
{
    (void)ByteLoader_POSIX_construct(this, fd);
}
ByteLoader_POSIX::ByteLoader_POSIX(const char* FileName)
{
    (void)ByteLoader_POSIX_construct_from_file_name(this, FileName); 
}


quex::ByteLoader*    
ByteLoader_POSIX_new(int fd)
{
    quex::ByteLoader_POSIX* me;

    if( fd == -1 ) return (quex::ByteLoader*)0;
    me = (quex::ByteLoader_POSIX*)quex::MemoryManager_allocate(sizeof(quex::ByteLoader_POSIX),
                                                           E_MemoryObjectType_BYTE_LOADER);
    if( ! me ) {
        return (quex::ByteLoader*)0;
    }
    else if( ! quex::ByteLoader_POSIX_construct(me, fd) ) {
        quex::MemoryManager_free(me, E_MemoryObjectType_BYTE_LOADER);
        return (quex::ByteLoader*)0;
    }
    else {
        (*me).ownership = E_Ownership_LEXER;
        return &(*me);
    }
}

quex::ByteLoader*    
ByteLoader_POSIX_new_from_file_name(const char* FileName)
{
    quex::ByteLoader_POSIX* me;

    me = (quex::ByteLoader_POSIX*)quex::MemoryManager_allocate(sizeof(quex::ByteLoader_POSIX),
                                                           E_MemoryObjectType_BYTE_LOADER);
    if( ! me ) {
        return (quex::ByteLoader*)0;
    }
    else if( ! quex::ByteLoader_POSIX_construct_from_file_name(me, FileName) ) {
        quex::MemoryManager_free(me, E_MemoryObjectType_BYTE_LOADER);
        return (quex::ByteLoader*)0;
    }
    else {
        (*me).ownership = E_Ownership_LEXER;
        return &(*me);
    }
}

bool
ByteLoader_POSIX_construct_from_file_name(quex::ByteLoader_POSIX* me, 
                                                         const char*                       FileName)
{
    int fd = open(FileName, O_RDONLY);
    return quex::ByteLoader_POSIX_construct(me, fd);
}

bool
ByteLoader_POSIX_construct(quex::ByteLoader_POSIX* me, int fd)
{
    QUEX_TYPE_STREAM_POSITION chunk_position_zero = (fd != -1) ? (QUEX_TYPE_STREAM_POSITION)lseek(fd, 0, SEEK_CUR) 
                                                               : (QUEX_TYPE_STREAM_POSITION)0;
    me->fd = fd;

    /* A POSIX file handle is always in binary mode.                         */
    quex::ByteLoader_construct(&(*me), true,
                                    /* ChunkSizeInBytes */ 1,
                                    /* ChunkPositionOfReadByteIdxZero */ chunk_position_zero,
                                    quex::ByteLoader_POSIX_chunk_seek,
                                    quex::ByteLoader_POSIX_chunk_load,
                                    quex::ByteLoader_POSIX_destruct,
                                    quex::ByteLoader_POSIX_print_this,
                                    quex::ByteLoader_POSIX_compare_handle);

    return fd == -1 ?  false : true;
}

void    
ByteLoader_POSIX_destruct(quex::ByteLoader* alter_ego)
{
    quex::ByteLoader_POSIX* me = (quex::ByteLoader_POSIX*)(alter_ego);

    if( me->fd != -1 ) {
        close(me->fd);
        me->fd = -1;
    }
}

void                      
ByteLoader_POSIX_chunk_seek(quex::ByteLoader*    alter_ego, 
                                 QUEX_TYPE_STREAM_POSITION Pos) 
{ 
    quex::ByteLoader_POSIX* me = (quex::ByteLoader_POSIX*)(alter_ego);
    if( me->fd == -1 ) { return; }
    (void)lseek(me->fd, (long)Pos, SEEK_SET); 
}

size_t  
ByteLoader_POSIX_chunk_load(quex::ByteLoader* alter_ego, 
                                     void*                       buffer, 
                                     const size_t                ByteN, 
                                     bool*                       end_of_stream_f) 
/* The POSIX interface does not allow to detect end of file upon reading.
 * The caller will realize end of stream by a return of zero bytes.          */
{ 
    quex::ByteLoader_POSIX* me = (quex::ByteLoader_POSIX*)(alter_ego);
    ssize_t                      n;

    if( me->fd == -1 ) { *end_of_stream_f = true; return 0; }

    n = read(me->fd, buffer, ByteN); 

    /* Theoretically, a last 'terminating zero' might be send over socket 
     * connections. Make sure, that this does not appear in the stream.      */
    if( n && ((uint8_t*)buffer)[n-1] == 0x0 ) {
        --n;
    }
    *end_of_stream_f = false;
    return (size_t)n;
}

bool  
ByteLoader_POSIX_compare_handle(const quex::ByteLoader* alter_ego_A, 
                                           const quex::ByteLoader* alter_ego_B) 
/* RETURNS: true  -- if A and B point to the same POSIX object.
 *          false -- else.                                                   */
{ 
    const quex::ByteLoader_POSIX* A = (quex::ByteLoader_POSIX*)(alter_ego_A);
    const quex::ByteLoader_POSIX* B = (quex::ByteLoader_POSIX*)(alter_ego_B);

    return A->fd == B->fd;
}

void                       
ByteLoader_POSIX_print_this(quex::ByteLoader* alter_ego)
{
    quex::ByteLoader_POSIX* me = (quex::ByteLoader_POSIX*)(alter_ego);

    QUEX_DEBUG_PRINT("        type:             POSIX;\n");
    QUEX_DEBUG_PRINT1("        file_descriptor:  ((%i));\n", (int)me->fd);
    QUEX_DEBUG_PRINT("        end_of_stream_f:  <no means to detect>;\n");
}

} /* close quex */

#endif /*  QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_POSIX_I */

