/* -*- C++ -*- vim: set syntax=cpp: */
#ifndef  QUEX_INCLUDE_GUARD_TestAnalyzer__BUFFER__LEXATOMS__LEXATOM_LOADER_I
#define  QUEX_INCLUDE_GUARD_TestAnalyzer__BUFFER__LEXATOMS__LEXATOM_LOADER_I

#include "test_cpp_cg/lib/definitions"
#include "test_cpp_cg/lib/buffer/Buffer"
#include "test_cpp_cg/lib/buffer/lexatoms/LexatomLoader"
#include "test_cpp_cg/lib/buffer/Buffer_print"
#include "test_cpp_cg/lib/quex/MemoryManager"



inline bool       TestAnalyzer_LexatomLoader_lexatom_index_seek(TestAnalyzer_LexatomLoader*         me, 
                                                                   const QUEX_TYPE_STREAM_POSITION  LexatomIndex);
inline ptrdiff_t  TestAnalyzer_LexatomLoader_load(TestAnalyzer_LexatomLoader*  me, 
                                                     TestAnalyzer_lexatom_t*         RegionBeginP, 
                                                     const ptrdiff_t            Size,
                                                     QUEX_TYPE_STREAM_POSITION  CharacterIndexBegin,
                                                     bool*                      end_of_stream_f,
                                                     bool*                      encoding_error_f);
inline bool       TestAnalyzer_LexatomLoader_lexatom_index_step_to(TestAnalyzer_LexatomLoader*        me,
                                                                      const QUEX_TYPE_STREAM_POSITION TargetCI);
inline void       TestAnalyzer_LexatomLoader_reverse_byte_order(TestAnalyzer_lexatom_t*       Begin, 
                                                                  const TestAnalyzer_lexatom_t* End);
inline void       TestAnalyzer_LexatomLoader_destruct(TestAnalyzer_LexatomLoader*); 
inline bool       TestAnalyzer_LexatomLoader_ByteLoader_Converter_consistency(quex::ByteLoader*  byte_loader, 
                                                                                 quex::Converter*   converter);


                       
inline TestAnalyzer_LexatomLoader*
TestAnalyzer_LexatomLoader_new(quex::ByteLoader*  byte_loader, 
                             quex::Converter*   converter)
{
    TestAnalyzer_LexatomLoader* filler;

    /* byte_loader = 0; possible if memory is filled manually.               */
    if( converter ) {
        if( ! TestAnalyzer_LexatomLoader_ByteLoader_Converter_consistency(byte_loader, converter) ) {
            return (TestAnalyzer_LexatomLoader*)0;
        }
        filler = TestAnalyzer_LexatomLoader_Converter_new(byte_loader, converter);
    }
    else {
        filler = TestAnalyzer_LexatomLoader_Plain_new(byte_loader); 
    }
    
    return filler;
}

inline bool
TestAnalyzer_LexatomLoader_ByteLoader_Converter_consistency(quex::ByteLoader*  byte_loader, 
                                                          quex::Converter*   converter)
{
    if( ! byte_loader ) {
        return true;
    }
    else if( converter->input_code_unit_size == (size_t)-1 ) {
        return true;
    }
    else if( converter->input_code_unit_size >= (size_t)byte_loader->chunk_size_in_bytes ) {
        return true;
    }
    else {
        QUEX_DEBUG_PRINT1("Error: The specified byte loader provides elements of size %i.\n", 
                          (int)byte_loader->chunk_size_in_bytes);
        QUEX_DEBUG_PRINT1("Error: The converter requires input elements of size <= %i.\n", 
                          (int)converter->input_code_unit_size);
        QUEX_DEBUG_PRINT("Error: This happens, for example, when using 'wistream' input\n"
                         "Error: without considering 'sizeof(wchar_t)' with respect to\n"
                         "Error: the encodings code unit's size. (UTF8=1byte, UTF16=2byte, etc.)\n");
        return false;
    }
}

inline void       
TestAnalyzer_LexatomLoader_destruct(TestAnalyzer_LexatomLoader* me)
{ 
    if( ! me ) return;

    if( me->byte_loader ) {
        quex::ByteLoader_delete(&me->byte_loader);
    }

    /* destruct_self: Free resources occupied by 'me' BUT NOT 'myself'.
     * delete_self:   Free resources occupied by 'me' AND 'myself'.           */
    if( me->derived.destruct ) {
        me->derived.destruct(me);
    }
}

inline void    
TestAnalyzer_LexatomLoader_setup(TestAnalyzer_LexatomLoader*   me,
                               size_t       (*derived_load_lexatoms)(TestAnalyzer_LexatomLoader*,
                                                                     TestAnalyzer_lexatom_t*, 
                                                                     const size_t, 
                                                                     bool*, bool*),
                               ptrdiff_t    (*stomach_byte_n)(TestAnalyzer_LexatomLoader*),
                               void         (*stomach_clear)(TestAnalyzer_LexatomLoader*),
                               void         (*derived_destruct)(TestAnalyzer_LexatomLoader*),
                               void         (*derived_fill_prepare)(TestAnalyzer_LexatomLoader*  me,
                                                                    TestAnalyzer_Buffer*         buffer,
                                                                    void**                     begin_p,
                                                                    const void**               end_p),
                               ptrdiff_t    (*derived_fill_finish)(TestAnalyzer_LexatomLoader* me,
                                                                   TestAnalyzer_lexatom_t*        BeginP,
                                                                   const TestAnalyzer_lexatom_t*  EndP,
                                                                   const void*               FilledEndP),
                               void         (*derived_get_fill_boundaries)(TestAnalyzer_LexatomLoader*  alter_ego,
                                                                           TestAnalyzer_Buffer*         buffer,
                                                                           void**                     begin_p, 
                                                                           const void**               end_p),
                               void         (*derived_print_this)(TestAnalyzer_LexatomLoader*  alter_ego),
                               quex::ByteLoader*  byte_loader,
                               ptrdiff_t    ByteNPerCharacter)
{
    __quex_assert(0 != me);
    __quex_assert(0 != derived_load_lexatoms);
    __quex_assert(0 != derived_destruct);

    /* Support for buffer filling without user interaction                   */
    me->stomach_byte_n        = stomach_byte_n;
    me->stomach_clear         = stomach_clear;
    me->load                  = TestAnalyzer_LexatomLoader_load;
    me->derived.load_lexatoms = derived_load_lexatoms;
    me->derived.destruct      = derived_destruct;
    me->derived.print_this    = derived_print_this;
    me->destruct              = TestAnalyzer_LexatomLoader_destruct;

    /* Support for manual buffer filling.                                    */
    me->derived.fill_prepare        = derived_fill_prepare;
    me->derived.fill_finish         = derived_fill_finish;
    me->derived.get_fill_boundaries = derived_get_fill_boundaries;

    me->byte_loader                 = byte_loader;

    me->_byte_order_reversion_active_f = false;
    me->lexatom_index_next_to_fill   = 0;
    me->byte_n_per_lexatom           = ByteNPerCharacter;
}

inline ptrdiff_t       
TestAnalyzer_LexatomLoader_load(TestAnalyzer_LexatomLoader*  me, 
                              TestAnalyzer_lexatom_t*         LoadP, 
                              const ptrdiff_t            LoadN,
                              QUEX_TYPE_STREAM_POSITION  StartLexatomIndex,
                              bool*                      end_of_stream_f,
                              bool*                      encoding_error_f)
/* Seeks the input position StartLexatomIndex and loads 'LoadN' 
 * lexatoms into the engine's buffer starting from 'LoadP'.
 *
 * RETURNS: Number of loaded lexatoms.                                       */
{
    ptrdiff_t                loaded_n;

    /* (1) Seek to the position where loading shall start.                       
     *                                                                       */
    if( ! TestAnalyzer_LexatomLoader_lexatom_index_seek(me, StartLexatomIndex) ) {
        return 0;
    }
    __quex_assert(me->lexatom_index_next_to_fill == StartLexatomIndex);

    /* (2) Load content into the given region.                                   
     *                                                                       */
    loaded_n = (ptrdiff_t)me->derived.load_lexatoms(me, LoadP, (size_t)LoadN,
                                                    end_of_stream_f, encoding_error_f);
#   ifdef QUEX_OPTION_ASSERTS
    {
        const TestAnalyzer_lexatom_t* p;
        /* The buffer limit code is not to appear inside the loaded content. */
        for(p=LoadP; p != &LoadP[loaded_n]; ++p) {
            __quex_assert(*p != QUEX_TestAnalyzer_SETTING_BUFFER_LEXATOM_BUFFER_BORDER);
        }
    }
#   endif

    __quex_assert(loaded_n <= LoadN);
    me->lexatom_index_next_to_fill += loaded_n;

    /* (3) Optionally reverse the byte order.                                    
     *                                                                       */
    if( me->_byte_order_reversion_active_f ) {
        TestAnalyzer_LexatomLoader_reverse_byte_order(LoadP, &LoadP[loaded_n]);
    }

    return loaded_n;
}

inline void
TestAnalyzer_LexatomLoader_reverse_byte_order(TestAnalyzer_lexatom_t*       Begin, 
                                           const TestAnalyzer_lexatom_t* End)
{
    uint8_t              tmp = 0xFF;
    TestAnalyzer_lexatom_t* iterator = 0x0;

    switch( sizeof(TestAnalyzer_lexatom_t) ) {
    default:
        __quex_assert(false);
        break;
    case 1:
        /* Nothing to be done */
        break;
    case 2:
        for(iterator=Begin; iterator != End; ++iterator) {
            tmp = *(((uint8_t*)iterator) + 0);
            *(((uint8_t*)iterator) + 0) = *(((uint8_t*)iterator) + 1);
            *(((uint8_t*)iterator) + 1) = tmp;
        }
        break;
    case 4:
        for(iterator=Begin; iterator != End; ++iterator) {
            tmp = *(((uint8_t*)iterator) + 0);
            *(((uint8_t*)iterator) + 0) = *(((uint8_t*)iterator) + 3);
            *(((uint8_t*)iterator) + 3) = tmp;
            tmp = *(((uint8_t*)iterator) + 1);
            *(((uint8_t*)iterator) + 1) = *(((uint8_t*)iterator) + 2);
            *(((uint8_t*)iterator) + 2) = tmp;
        }
        break;
    }
}

inline void       
TestAnalyzer_LexatomLoader_print_this(TestAnalyzer_LexatomLoader* me)
{
    QUEX_DEBUG_PRINT("    filler: {\n");
    if( ! me ) {
        QUEX_DEBUG_PRINT("      type: <none>\n");
    }
    else {
        QUEX_DEBUG_PRINT1("      lexatom_index_next_to_fill:     %i;\n", 
                          (int)me->lexatom_index_next_to_fill);
        QUEX_DEBUG_PRINT1("      byte_n_per_lexatom:             %i;\n", 
                          (int)me->byte_n_per_lexatom);
        QUEX_DEBUG_PRINT1("      _byte_order_reversion_active_f: %s;\n", 
                          E_Boolean_NAME(me->_byte_order_reversion_active_f)); 
        /* me->byte_loader->print_this(me->byte_loader); */
        if( me->derived.print_this ) me->derived.print_this(me);
        if( ! me->byte_loader ) {
            QUEX_DEBUG_PRINT("      byte_loader: <none>\n");
        }
        else {
            if( me->byte_loader->print_this ) me->byte_loader->print_this(me->byte_loader); 
        }
    }
    QUEX_DEBUG_PRINT("    }\n");
}



#include "test_cpp_cg/lib/buffer/Buffer.i"
#include "test_cpp_cg/lib/buffer/lexatoms/LexatomLoader_navigation.i"
#include "test_cpp_cg/lib/buffer/lexatoms/LexatomLoader_Converter.i"
#include "test_cpp_cg/lib/buffer/lexatoms/LexatomLoader_Plain.i"

#endif /* QUEX_INCLUDE_GUARD_TestAnalyzer__BUFFER__BUFFERFILLER_I */

