/* vim: set ft=c:
 * (C) Frank-Rene Schaefer */
#ifndef  QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_FILE_I
#define  QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_FILE_I

#include "test_cpp_emm/lib/quex/MemoryManager"
#include "test_cpp_emm/lib/quex/byte_loader/ByteLoader_FILE"

namespace quex {

extern void                       ByteLoader_FILE_chunk_seek(quex::ByteLoader* me, QUEX_TYPE_STREAM_POSITION Pos);
extern size_t                     ByteLoader_FILE_chunk_load(quex::ByteLoader* me, void* buffer, const size_t ByteN, bool*);
extern void                       ByteLoader_FILE_destruct(quex::ByteLoader* me);
extern void                       ByteLoader_FILE_print_this(quex::ByteLoader* me);
extern bool                       ByteLoader_FILE_compare_handle(const quex::ByteLoader* alter_ego_A, 
                                                                                 const quex::ByteLoader* alter_ego_B);


ByteLoader_FILE::ByteLoader_FILE(FILE* fh, bool BinaryModeF)
{
    (void)ByteLoader_FILE_construct(this, fh, BinaryModeF);
}
ByteLoader_FILE::ByteLoader_FILE(const char* FileName)
{
    (void)ByteLoader_FILE_construct_from_file_name(this, FileName); 
}


quex::ByteLoader*    
ByteLoader_FILE_new(FILE* fh, bool BinaryModeF)
{
    quex::ByteLoader_FILE* me;
   
    me = (quex::ByteLoader_FILE*)quex::MemoryManager_allocate(sizeof(quex::ByteLoader_FILE),
                                                                     E_MemoryObjectType_BYTE_LOADER);
    if( ! me ) {
        return (quex::ByteLoader*)0;
    } 
    else if( ! quex::ByteLoader_FILE_construct(me, fh, BinaryModeF) ) {
        quex::MemoryManager_free(me, E_MemoryObjectType_BYTE_LOADER);
        return (quex::ByteLoader*)0;
    }
    else {
        (*me).ownership = E_Ownership_LEXER;
        return &(*me);
    }
}

quex::ByteLoader*    
ByteLoader_FILE_new_from_file_name(const char* FileName)
{
    quex::ByteLoader_FILE* me;

    me = (quex::ByteLoader_FILE*)quex::MemoryManager_allocate(sizeof(quex::ByteLoader_FILE),
                                                                     E_MemoryObjectType_BYTE_LOADER);
    if( ! me ) {
        return (quex::ByteLoader*)0;
    }
    else if( ! quex::ByteLoader_FILE_construct_from_file_name(me, FileName) ) {
        quex::MemoryManager_free(me, E_MemoryObjectType_BYTE_LOADER);
        return (quex::ByteLoader*)0;
    }
    else {
        (*me).ownership = E_Ownership_LEXER;
        return &(*me);
    }
}

bool
ByteLoader_FILE_construct_from_file_name(quex::ByteLoader_FILE* me, 
                                                        const char*                      FileName)
{
    FILE*  fh = fopen(FileName, "rb");

    return quex::ByteLoader_FILE_construct(me, fh, true);
}

bool
ByteLoader_FILE_construct(quex::ByteLoader_FILE* me, FILE* fh, bool BinaryModeF)
{
    /* IMPORTANT: input_handle must be set BEFORE call to base constructor!
     *            Constructor does call 'tell()'                             */
    QUEX_TYPE_STREAM_POSITION chunk_position_zero = fh ? (QUEX_TYPE_STREAM_POSITION )ftell(fh) 
                                                       : (QUEX_TYPE_STREAM_POSITION )0;
    me->input_handle = fh;

    quex::ByteLoader_construct(&(*me), BinaryModeF,
                                         /* ChunkSizeInBytes */ 1,
                                         /* ChunkPositionOfReadByteIdxZero */ chunk_position_zero,
                                         quex::ByteLoader_FILE_chunk_seek,
                                         quex::ByteLoader_FILE_chunk_load,
                                         quex::ByteLoader_FILE_destruct,
                                         quex::ByteLoader_FILE_print_this,
                                         quex::ByteLoader_FILE_compare_handle);
    return fh ? true : false;
}

void    
ByteLoader_FILE_destruct(quex::ByteLoader* alter_ego)
{
    quex::ByteLoader_FILE* me = (quex::ByteLoader_FILE*)(alter_ego);

    if( me->input_handle ) {
        fclose(me->input_handle);
        me->input_handle = (FILE*)0;
    }
}

void                      
ByteLoader_FILE_chunk_seek(quex::ByteLoader* alter_ego, QUEX_TYPE_STREAM_POSITION Pos) 
{ 
    quex::ByteLoader_FILE* me = (quex::ByteLoader_FILE*)(alter_ego);
    if( ! me->input_handle ) return;
    (void)fseek(me->input_handle, (long)Pos, SEEK_SET); 
}

size_t  
ByteLoader_FILE_chunk_load(quex::ByteLoader* alter_ego, 
                                void*                  buffer, 
                                const size_t           ByteN, 
                                bool*                  end_of_stream_f) 
{ 
    quex::ByteLoader_FILE* me = (quex::ByteLoader_FILE*)(alter_ego);
    size_t                      loaded_byte_n;

    if( ! me->input_handle ) { *end_of_stream_f = true; return 0; }

    loaded_byte_n = fread(buffer, 1, ByteN, me->input_handle); 
    *end_of_stream_f = feof(me->input_handle) ? true : false;
    return loaded_byte_n;
}

bool  
ByteLoader_FILE_compare_handle(const quex::ByteLoader* alter_ego_A, 
                                          const quex::ByteLoader* alter_ego_B) 
/* RETURNS: true  -- if A and B point to the same FILE object.
 *          false -- else.                                                   */
{ 
    const quex::ByteLoader_FILE* A = (quex::ByteLoader_FILE*)(alter_ego_A);
    const quex::ByteLoader_FILE* B = (quex::ByteLoader_FILE*)(alter_ego_B);

    return A->input_handle == B->input_handle;
}

void                       
ByteLoader_FILE_print_this(quex::ByteLoader* alter_ego)
{
    quex::ByteLoader_FILE* me = (quex::ByteLoader_FILE*)(alter_ego);

    QUEX_DEBUG_PRINT("        type:             FILE;\n");
    QUEX_DEBUG_PRINT1("        file_handle:      ((%p));\n", (const void*)me->input_handle);
    if( me->input_handle ) {
        QUEX_DEBUG_PRINT1("        end_of_stream:    %s;\n", E_Boolean_NAME(feof(me->input_handle)));
    }
}

} /* close quex */

#endif /*  QUEX_INCLUDE_GUARD_LIB_QUEX__BYTE_LOADER__BYTE_LOADER_FILE_I */
